"""

This code is copied from plone.openid 
http://svn.plone.org/svn/plone/plone.openid/trunk/plone/openid/tests/consumer.py

"""
from openid.consumer.consumer import FAILURE, SUCCESS

class MockAuthRequest:
    """Amock OpenID AuthRequest.
    """
    def __init__(self, status=None, identity_url=None, message=None):
        self.status=status
        self.identity_url=identity_url
        self.message=message


    def redirectURL(self, trust_root, return_to):
        return "http://someopenidprovider.com/somewhere"

    def getDisplayIdentifier(self):
        return "http://foobar.com"


class MockConsumer:
    """A mock OpenID consumerclass.
    """

    def begin(self, identity):
        self.identity=identity
        return MockAuthRequest()

    def complete(self, credentials, current_url):
        status=SUCCESS
        message="authentication completed succesfully"
        
        if credentials.has_key("openid.identity") and credentials["openid.identity"] == "":
            # if the python openid is passed an identity of an empty string
            # an IndexError is raised in the depths of its XRI identification
            # see: http://www.oasis-open.org/committees/tc_home.php?wg_abbrev=xri

            # an empty string is common when the submit button of the
            # openid login is clicked prior to providing an identity url
            # we simulate openid's response here in our mock object
            message="invalid identity"
            status=FAILURE
        else:
            for field in [ "nonce", "openid.identity",
                    "openid.assoc_handle", "openid.return_to", "openid.signed",
                    "openid.sig", "openid.invalidate_handle", "openid.mode"]:
                if field not in credentials:
                    message="field missing"
                    status=FAILURE
                

        return MockAuthRequest(status=status,
                                message=message,
                                identity_url=credentials["openid.identity"])

def get_consumer(environ):
    return MockConsumer()

def patch_plugin(plugin):
    plugin.get_consumer = get_consumer
    return plugin
    





# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2008-2009, Gustavo Narea <me@gustavonarea.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the BSD-like license at
# http://www.repoze.org/LICENSE.txt.  A copy of the license should accompany
# this distribution.  THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL
# EXPRESS OR IMPLIED WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO,
# THE IMPLIED WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND
# FITNESS FOR A PARTICULAR PURPOSE
#
##############################################################################

"""Mock Elixir-powered model definition."""

from hashlib import sha1
from datetime import datetime

from sqlalchemy.orm import scoped_session, sessionmaker
import elixir
from elixir import Entity, Field
from elixir import DateTime, Unicode
from elixir import using_options

DBSession = scoped_session(sessionmaker(autoflush=True, autocommit=False))

metadata = elixir.metadata
elixir.session = DBSession

def init_model(engine):
    """Call me before using any of the tables or classes in the model."""
    DBSession.configure(bind=engine)
    metadata.bind = engine


class User(Entity):
    """Reasonably basic User definition. Probably would want additional
    attributes.
    """
    using_options(tablename="user", auto_primarykey="user_id")

    user_name = Field(Unicode(16), required=True, unique=True)

    _password = Field(Unicode(40), colname="password", required=True)

    def _set_password(self, password):
        """encrypts password on the fly"""
        self._password = self.__encrypt_password(password)

    def _get_password(self):
        """returns password"""
        return self._password

    password = descriptor=property(_get_password, _set_password)

    def __encrypt_password(self, password):
        """Hash the given password with SHA1."""
        
        if isinstance(password, unicode):
            password_8bit = password.encode('UTF-8')

        else:
            password_8bit = password

        hashed_password = sha1()
        hashed_password.update(password_8bit)
        hashed_password = hashed_password.hexdigest()

        # make sure the hased password is an UTF-8 object at the end of the
        # process because SQLAlchemy _wants_ a unicode object for Unicode columns
        if not isinstance(hashed_password, unicode):
            hashed_password = hashed_password.decode('UTF-8')

        return hashed_password

    def validate_password(self, password):
        """Check the password against existing credentials.
        this method _MUST_ return a boolean.

        @param password: the password that was provided by the user to
        try and authenticate. This is the clear text version that we will
        need to match against the (possibly) encrypted one in the database.
        @type password: unicode object
        """
        return self.password == self.__encrypt_password(password)



# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009, Gustavo Narea <me@gustavonarea.net>.
# All Rights Reserved.
#
# This software is subject to the provisions of the BSD-like license at
# http://www.repoze.org/LICENSE.txt.  A copy of the license should accompany
# this distribution.  THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL
# EXPRESS OR IMPLIED WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO,
# THE IMPLIED WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND
# FITNESS FOR A PARTICULAR PURPOSE.
#
##############################################################################

"""Test utilities for repoze.who-powered applications."""

import sys
from logging import INFO
from re import compile as compile_regex

from zope.interface import implements
from paste.httpexceptions import HTTPUnauthorized
from paste.deploy.converters import asbool

from repoze.who.middleware import PluggableAuthenticationMiddleware
from repoze.who.config import WhoConfig, \
                              make_middleware_with_config as mk_mw_cfg
from repoze.who.interfaces import IIdentifier, IAuthenticator, IChallenger

__all__ = ['AuthenticationForgerPlugin', 'AuthenticationForgerMiddleware',
           'make_middleware', 'make_middleware_with_config']


_HTTP_STATUS_PATTERN = compile_regex(r'^(?P<code>[0-9]{3}) (?P<reason>.*)$')


class AuthenticationForgerPlugin(object):
    """
    :mod:`repoze.who` plugin to forge authentication easily and bypass
    :mod:`repoze.who` challenges.
    
    This plugin enables you to write identifier and challenger-independent
    tests. As a result, your protected areas will be easier to test:
    
    #. To forge authentication, without bypassing identification (i.e., running
       MD providers), you can use the following WebTest-powered test::
       
           def test_authorization_granted(self):
               '''The right subject must get what she requested'''
               environ = {'REMOTE_USER': 'manager'}
               resp = self.app.get('/admin/', extra_environ=environ, status=200)
               assert 'some text' in resp.body
       
       As you can see, this is an identifier-independent way to forge
       authentication.
    
    #. To check that authorization was denied, in a challenger-independent way,
       you can use::
       
           def test_authorization_denied_anonymous(self):
               '''Anonymous users must get a 401 page'''
               self.app.get('/admin/', status=401)
       
           def test_authorization_denied_authenticated(self):
               '''Authenticated users must get a 403 page'''
               environ = {'REMOTE_USER': 'editor'}
               self.app.get('/admin/', extra_environ=environ, status=403)
    
    """
    
    implements(IIdentifier, IAuthenticator, IChallenger)
    
    def __init__(self, fake_user_key='REMOTE_USER',
                 remote_user_key='repoze.who.testutil.userid'):
        """
        
        :param fake_user_key: The key for the item in the ``environ`` which
            will contain the forged user Id.
        :type fake_user_key: str
        :param remote_user_key: The actual "external" ``remote_user_key``
            used by :mod:`repoze.who`.
        :type remote_user_key: str 
        
        """
        self.fake_user_key = fake_user_key
        self.remote_user_key = remote_user_key
    
    # IIdentifier
    def identify(self, environ):
        """
        Pre-authenticate using the user Id found in the relevant ``environ``
        item, if any.
        
        The user Id. found will be put into ``identity['fake-userid']``, for
        :meth:`authenticate`.
        
        """
        if self.fake_user_key in environ:
            identity = {'fake-userid': environ[self.fake_user_key]}
            return identity
    
    # IIdentifier
    def remember(self, environ, identity):
        """Do nothing"""
        pass
    
    # IIdentifier
    def forget(self, environ, identity):
        """Do nothing"""
        pass
    
    # IAuthenticator
    def authenticate(self, environ, identity):
        """
        Turn the value in ``identity['fake-userid']`` into the remote user's
        name.
        
        Finally, it removes ``identity['fake-userid']`` so that it won't reach
        the WSGI application.
        
        """
        if 'fake-userid' in identity:
            environ[self.remote_user_key] = identity.pop('fake-userid')
            return environ[self.remote_user_key]
    
    # IChallenger
    def challenge(self, environ, status, app_headers, forget_headers):
        """Return a 401 page unconditionally."""
        headers = app_headers + forget_headers
        # The HTTP status code and reason may not be the default ones:
        status_parts = _HTTP_STATUS_PATTERN.search(status)
        if status_parts:
            reason = status_parts.group('reason')
            code = int(status_parts.group('code'))
        else:
            reason = 'HTTP Unauthorized'
            code = 401
        # Building the response:
        response = HTTPUnauthorized(headers=headers)
        response.title = reason
        response.code = code
        return response


class AuthenticationForgerMiddleware(PluggableAuthenticationMiddleware):
    """
    :class:`PluggableAuthenticationMiddleware
    <repoze.who.middleware.PluggableAuthenticationMiddleware>` proxy
    to forge authentication, without bypassing identification.
    
    """
    
    def __init__(self, app, identifiers, authenticators, challengers,
                 mdproviders, classifier, challenge_decider, log_stream=None,
                 log_level=INFO, remote_user_key='REMOTE_USER'):
        """
        Setup authentication in an easy to forge way.
        
        All the arguments received will be passed as is to
        :class:`repoze.who.middleware.PluggableAuthenticationMiddleware`,
        with one instance of :class:`AuthenticationForgerPlugin` in:
        
        * ``identifiers``. This instance will be inserted in the first position
          of the list.
        * ``authenticators``. Any authenticator passed will be ignored; such
          an instance will be the only authenticator defined.
        * ``challengers``. Any challenger passed will be ignored; such
          an instance will be the only challenger defined.
        
        Internally, it will also set ``remote_user_key`` to
        ``'repoze.who.testutil.userid'``, so that you can use the standard
        ``'REMOTE_USER'`` in your tests.
        
        The metadata providers won't be modified.
        
        """
        
        self.actual_remote_user_key = remote_user_key
        forger = AuthenticationForgerPlugin(fake_user_key=remote_user_key)
        forger = ('auth_forger', forger)
        identifiers.insert(0, forger)
        authenticators = [forger]
        challengers = [forger]
        # Calling the parent's constructor:
        init = super(AuthenticationForgerMiddleware, self).__init__
        init(app, identifiers, authenticators, challengers, mdproviders,
             classifier, challenge_decider, log_stream, log_level,
             'repoze.who.testutil.userid')


#{ Middleware makers:


def make_middleware(skip_authentication=False, *args, **kwargs):
    """
    Return the requested authentication middleware.
    
    :param skip_authentication: If ``True``, an instance of
        :class:`AuthenticationForgerMiddleware` will be returned instead of
        :class:`repoze.who.middleware.PluggableAuthenticationMiddleware`
    :type skip_authentication: bool
    
    ``args`` and ``kwargs`` are the positional and named arguments, 
    respectively, to be passed to the relevant authentication middleware.
    
    """
    
    if asbool(skip_authentication):
        # We must replace the middleware:
        return AuthenticationForgerMiddleware(*args, **kwargs)
    else:
        return PluggableAuthenticationMiddleware(*args, **kwargs)


def make_middleware_with_config(app, global_conf, config_file, log_file=None,
                                log_level=None, skip_authentication=False):
    """
    Proxy :func:`repoze.who.config.make_middleware_with_config` to skip
    authentication when required.
    
    If  ``skip_authentication`` evaluates to ``True``, then the returned 
    middleware will be an instance of :class:`AuthenticationForgerMiddleware`.
    
    """
    if not asbool(skip_authentication):
        # We must not replace the middleware
        return mk_mw_cfg(app, global_conf, config_file, log_file, log_level)
    
    # We must replace the middleware:
    parser = WhoConfig(global_conf['here'])
    parser.parse(open(config_file))
    return AuthenticationForgerMiddleware(
        app,
        parser.identifiers,
        parser.authenticators,
        parser.challengers,
        parser.mdproviders,
        parser.request_classifier,
        parser.challenge_decider,
        remote_user_key=parser.remote_user_key,
        )

#}



# -*- coding: utf-8 -*-
#
# repoze.who.plugins.ldap, LDAP authentication for WSGI applications.
# Copyright (C) 2008 by Gustavo Narea <http://gustavonarea.net/>
#
# This file is part of repoze.who.plugins.ldap
# <http://code.gustavonarea.net/repoze.who.plugins.ldap/>
#
# repoze.who.plugins.ldap is freedomware: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by the
# Free Software Foundation, either version 3 of the License, or any later
# version.
#
# repoze.who.plugins.ldap is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of 
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# repoze.who.plugins.ldap. If not, see <http://www.gnu.org/licenses/>.

import os

from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, 'README')).read()
CHANGELOG = open(os.path.join(here, 'CHANGELOG')).read()
version = open(os.path.join(here, 'VERSION')).readline().rstrip()

setup(
    name='repoze.who.plugins.ldap',
    version=version,
    description="LDAP plugin for repoze.who",
    long_description='\n\n'.join([README, CHANGELOG]),
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU General Public License (GPL)",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Topic :: Internet :: WWW/HTTP :: WSGI :: Middleware",
        "Topic :: System :: Systems Administration :: Authentication/Directory :: LDAP"
    ],
    keywords='ldap web application server wsgi repoze repoze.who',
    author="Gustavo Narea",
    author_email="me@gustavonarea.net",
    url="http://code.gustavonarea.net/repoze.who.plugins.ldap/",
    download_url="https://launchpad.net/repoze.who.plugins.ldap/+download",
    license="GNU General Public License v3",
    include_package_data=True,
    packages=find_packages(exclude=["*.tests", "demo", "demo.*"]),
    namespace_packages=['repoze', 'repoze.who', 'repoze.who.plugins'],
    zip_safe=False,
    tests_require = ['dataflake.ldapconnection>=0.3'],
    install_requires=[
        'repoze.who>=1.0.6',
        'python-ldap>=2.3.5',
        'setuptools',
        'zope.interface'
        ],
    test_suite="repoze.who.plugins.ldap.tests.suite"
    )



# -*- coding: utf-8 -*-
#
# repoze.who.plugins.ldap, LDAP authentication for WSGI applications.
# Copyright (C) 2008 by Gustavo Narea <http://gustavonarea.net/>
#
# This file is part of repoze.who.plugins.ldap
# <http://code.gustavonarea.net/repoze.who.plugins.ldap/>
#
# repoze.who.plugins.ldap is freedomware: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by the
# Free Software Foundation, either version 3 of the License, or any later
# version.
#
# repoze.who.plugins.ldap is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of 
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# repoze.who.plugins.ldap. If not, see <http://www.gnu.org/licenses/>.

"""Test suite for repoze.who.plugins.ldap"""

import unittest

from dataflake.ldapconnection.tests import fakeldap
from ldap import modlist
from ldap.ldapobject import SimpleLDAPObject
from zope.interface.verify import verifyClass
from repoze.who.interfaces import IAuthenticator, IMetadataProvider

from repoze.who.plugins.ldap import LDAPAuthenticatorPlugin, \
                                    LDAPAttributesPlugin
from repoze.who.plugins.ldap.plugins import make_ldap_connection


class Base(unittest.TestCase):
    """Base test case for the plugins"""
    
    def setUp(self):
        # Connecting to a fake server with a fake account:
        conn = fakeldap.initialize('ldap://example.org')
        conn.simple_bind_s('Manager', 'some password')
        # Adding a fake user, which is used in the tests
        person_attr = {'cn': [fakeuser['cn']],
                       'uid': fakeuser['uid'],
                       'userPassword': [fakeuser['hashedPassword']]}
        conn.add_s(fakeuser['dn'], modlist.addModlist(person_attr))
        self.connection = conn
        # Creating a fake environment:
        self.env  = self._makeEnviron()
    
    def tearDown(self):
        self.connection.delete_s(fakeuser['dn'])
    
    def _makeEnviron(self, kw=None):
        """Create a fake WSGI environment
        
        This is based on the same method of the test suite of repoze.who.
        
        """
        environ = {}
        environ['wsgi.version'] = (1,0)
        if kw is not None:
            environ.update(kw)
        return environ


#{ Test cases for the plugins


class TestMakeLDAPAuthenticatorPlugin(unittest.TestCase):
    """Tests for the constructor of the L{LDAPAuthenticatorPlugin} plugin"""
    
    def test_without_connection(self):
        self.assertRaises(ValueError, LDAPAuthenticatorPlugin, None,
                          'dc=example,dc=org')
    def test_without_base_dn(self):
        conn = fakeldap.initialize('ldap://example.org')
        self.assertRaises(TypeError, LDAPAuthenticatorPlugin, conn)
        self.assertRaises(ValueError, LDAPAuthenticatorPlugin, conn, None)
    
    def test_with_connection(self):
        conn = fakeldap.initialize('ldap://example.org')
        LDAPAuthenticatorPlugin(conn, 'dc=example,dc=org')
    
    def test_connection_is_url(self):
        LDAPAuthenticatorPlugin('ldap://example.org', 'dc=example,dc=org')


class TestLDAPAuthenticatorPlugin(Base):
    """Tests for the L{LDAPAuthenticatorPlugin} IAuthenticator plugin"""
    
    def setUp(self):
        super(TestLDAPAuthenticatorPlugin, self).setUp()
        # Loading the plugin:
        self.plugin = LDAPAuthenticatorPlugin(self.connection, base_dn)

    def test_implements(self):
        verifyClass(IAuthenticator, LDAPAuthenticatorPlugin, tentative=True)

    def test_authenticate_nologin(self):
        result = self.plugin.authenticate(self.env, None)
        self.assertEqual(result, None)

    def test_authenticate_incomplete_credentials(self):
        identity1 = {'login': fakeuser['uid']}
        identity2 = {'password': fakeuser['password']}
        result1 = self.plugin.authenticate(self.env, identity1)
        result2 = self.plugin.authenticate(self.env, identity2)
        self.assertEqual(result1, None)
        self.assertEqual(result2, None)

    def test_authenticate_noresults(self):
        identity = {'login': 'i_dont_exist',
                    'password': 'super secure password'}
        result = self.plugin.authenticate(self.env, identity)
        self.assertEqual(result, None)

    def test_authenticate_comparefail(self):
        identity = {'login': fakeuser['uid'],
                    'password': 'wrong password'}
        result = self.plugin.authenticate(self.env, identity)
        self.assertEqual(result, None)

    def test_authenticate_comparesuccess(self):
        identity = {'login': fakeuser['uid'],
                    'password': fakeuser['password']}
        result = self.plugin.authenticate(self.env, identity)
        self.assertEqual(result, fakeuser['dn'])
    
    def test_custom_authenticator(self):
        """L{LDAPAuthenticatorPlugin._get_dn} should be overriden with no
        problems"""
        plugin = CustomLDAPAuthenticatorPlugin(self.connection, base_dn)
        identity = {'login': fakeuser['uid'],
                    'password': fakeuser['password']}
        result = plugin.authenticate(self.env, identity)
        expected = 'uid=%s,ou=admins,%s' % (fakeuser['uid'], base_dn)
        self.assertEqual(result, expected)


class TestMakeLDAPAttributesPlugin(unittest.TestCase):
    """Tests for the constructor of L{LDAPAttributesPlugin}"""
    
    def test_connection_is_invalid(self):
        self.assertRaises(ValueError, LDAPAttributesPlugin, None, 'cn')
    
    def test_attributes_is_none(self):
        """If attributes is None then fetch all the attributes"""
        plugin = LDAPAttributesPlugin('ldap://localhost', None)
        self.assertEqual(plugin.attributes, None)
    
    def test_attributes_is_comma_separated_str(self):
        attributes = "cn,uid,mail"
        plugin = LDAPAttributesPlugin('ldap://localhost', attributes)
        self.assertEqual(plugin.attributes, attributes.split(','))
    
    def test_attributes_is_only_one_as_str(self):
        attributes = "mail"
        plugin = LDAPAttributesPlugin('ldap://localhost', attributes)
        self.assertEqual(plugin.attributes, ['mail'])
    
    def test_attributes_is_iterable(self):
        # The plugin, with a tuple as attributes
        attributes_t = ('cn', 'mail')
        plugin_t = LDAPAttributesPlugin('ldap://localhost', attributes_t)
        self.assertEqual(plugin_t.attributes, list(attributes_t))
        # The plugin, with a list as attributes
        attributes_l = ['cn', 'mail']
        plugin_l = LDAPAttributesPlugin('ldap://localhost', attributes_l)
        self.assertEqual(plugin_l.attributes, attributes_l)
        # The plugin, with a dict as attributes
        attributes_d = {'first': 'cn', 'second': 'mail'}
        plugin_d = LDAPAttributesPlugin('ldap://localhost', attributes_d)
        self.assertEqual(plugin_d.attributes, list(attributes_d))
    
    def test_attributes_is_not_iterable_nor_string(self):
        self.assertRaises(ValueError, LDAPAttributesPlugin, 'ldap://localhost',
                          12345)
    
    def test_parameters_are_valid(self):
        LDAPAttributesPlugin('ldap://localhost', 'cn', '(objectClass=*)')


class TestLDAPAttributesPlugin(Base):
    """Tests for the L{LDAPAttributesPlugin} IMetadata plugin"""

    def test_implements(self):
        verifyClass(IMetadataProvider, LDAPAttributesPlugin, tentative=True)

    def test_add_metadata(self):
        plugin = LDAPAttributesPlugin(self.connection)
        environ = {}
        identity = {'repoze.who.userid': fakeuser['dn']}
        expected_identity = {
            'repoze.who.userid': fakeuser['dn'],
            'cn': [fakeuser['cn']],
            'userPassword': [fakeuser['hashedPassword']],
            'uid': fakeuser['uid']
        }
        plugin.add_metadata(environ, identity)
        self.assertEqual(identity, expected_identity)


# Test cases for plugin utilities


class TestLDAPConnectionFactory(unittest.TestCase):
    """Tests for L{make_ldap_connection}"""
    
    def test_connection_is_object(self):
        conn = fakeldap.initialize('ldap://example.org')
        self.assertEqual(make_ldap_connection(conn), conn)
    
    def test_connection_is_str(self):
        conn = make_ldap_connection('ldap://example.org')
        self.assertTrue(isinstance(conn, SimpleLDAPObject))
    
    def test_connection_is_unicode(self):
        conn = make_ldap_connection(u'ldap://example.org')
        self.assertTrue(isinstance(conn, SimpleLDAPObject))
    
    def test_connection_is_none(self):
        self.assertRaises(ValueError, make_ldap_connection, None)


#{ Fixtures

base_dn = 'ou=people,dc=example,dc=org'
    
fakeuser = {
    'dn': 'uid=carla,%s' % base_dn,
    'uid': 'carla',
    'cn': 'Carla Paola',
    'mail': 'carla@example.org',
    'password': 'hello',
    'hashedPassword': '{SHA}qvTGHdzF6KLavt4PO0gs2a6pQ00='
}

class CustomLDAPAuthenticatorPlugin(LDAPAuthenticatorPlugin):
    """Fake class to test that L{LDAPAuthenticatorPlugin._get_dn} can be
    overriden with no problems"""
    def _get_dn(self, environ, identity):
        try:
            return u'uid=%s,ou=admins,%s' % (identity['login'], self.base_dn)
        except (KeyError, TypeError):
            raise ValueError, ('Could not find the DN from the identity and '
                               'environment')


#}


def suite():
    """
    Return the test suite.
    
    @return: The test suite for the plugin.
    @rtype: C{unittest.TestSuite}
    
    """
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestMakeLDAPAuthenticatorPlugin, "test"))
    suite.addTest(unittest.makeSuite(TestLDAPAuthenticatorPlugin, "test"))
    suite.addTest(unittest.makeSuite(TestMakeLDAPAttributesPlugin, "test"))
    suite.addTest(unittest.makeSuite(TestLDAPAttributesPlugin, "test"))
    suite.addTest(unittest.makeSuite(TestLDAPConnectionFactory, "test"))
    return suite


if __name__ == '__main__':
    unittest.main(defaultTest='suite')



from ldapauth.lib.base import *

class TemplateController(BaseController):

    def view(self, url):
        """By default, the final controller tried to fulfill the request
        when no other routes match. It may be used to display a template
        when all else fails, e.g.::

            def view(self, url):
                return render('/%s' % url)

        Or if you're using Mako and want to explicitly send a 404 (Not
        Found) response code when the requested template doesn't exist::

            import mako.exceptions

            def view(self, url):
                try:
                    return render('/%s' % url)
                except mako.exceptions.TopLevelLookupException:
                    abort(404)

        By default this controller aborts the request with a 404 (Not
        Found)
        """
        abort(404)



"""TurboGears middleware initialization"""
from repoze.who.config import make_middleware_with_config as make_who_with_config

from ldapauth.config.app_cfg import base_config
from ldapauth.config.environment import load_environment

#Use base_config to setup the nessisary WSGI App factory. 
#make_base_app will wrap the TG2 app with all the middleware it needs. 
make_base_app = base_config.setup_tg_wsgi_app(load_environment)

def make_app(global_conf, full_stack=True, **app_conf):
    app = make_base_app(global_conf, full_stack=True, **app_conf)
    #Wrap your base turbogears app with custom middleware
    app = make_who_with_config(app, global_conf, app_conf['who.config_file'], 
                               app_conf['who.log_file'], 
                               app_conf['who.log_level'])
    return app





"""The base Controller API

Provides the BaseController class for subclassing.
"""
from tg import TGController, tmpl_context
from tg.render import render

import ldapauth.model as model

from pylons.i18n import _, ungettext, N_
from tw.api import WidgetBunch

class Controller(object):
    """Base class for a web application's controller.

    Currently, this provides positional parameters functionality
    via a standard default method.
    """

class BaseController(TGController):
    """Base class for the root of a web application.

    Your web application should have one of these. The root of
    your application is used to compute URLs used by your app.
    """

    def __call__(self, environ, start_response):
        """Invoke the Controller"""
        # TGController.__call__ dispatches to the Controller method
        # the request is routed to. This routing information is
        # available in environ['pylons.routes_dict']
        
        return TGController.__call__(self, environ, start_response)



import md5
import sha
from datetime import datetime

from tg import config
from sqlalchemy import Table, ForeignKey, Column
from sqlalchemy.types import String, Unicode, UnicodeText, Integer, DateTime, \
                             Boolean, Float
from sqlalchemy.orm import relation, backref, synonym

from ldapauth.model import DeclarativeBase, metadata, DBSession


# This is the association table for the many-to-many relationship between
# groups and permissions.
group_permission_table = Table('tg_group_permission', metadata,
    Column('group_id', Integer, ForeignKey('tg_group.group_id',
        onupdate="CASCADE", ondelete="CASCADE")),
    Column('permission_id', Integer, ForeignKey('tg_permission.permission_id',
        onupdate="CASCADE", ondelete="CASCADE"))
)

# This is the association table for the many-to-many relationship between
# groups and members - this is, the memberships.
user_group_table = Table('tg_user_group', metadata,
    Column('user_id', Integer, ForeignKey('tg_user.user_id',
        onupdate="CASCADE", ondelete="CASCADE")),
    Column('group_id', Integer, ForeignKey('tg_group.group_id',
        onupdate="CASCADE", ondelete="CASCADE"))
)

# identity model

class Group(DeclarativeBase):
    """An ultra-simple group definition.
    """
    __tablename__ = 'tg_group'

    group_id = Column(Integer, autoincrement=True, primary_key=True)

    group_name = Column(Unicode(16), unique=True)

    display_name = Column(Unicode(255))

    created = Column(DateTime, default=datetime.now)

    users = relation('User', secondary=user_group_table, backref='groups')

    def __repr__(self):
        return '<Group: name=%s>' % self.group_name


class User(DeclarativeBase):
    """Reasonably basic User definition. Probably would want additional
    attributes.
    """
    __tablename__ = 'tg_user'

    user_id = Column(Integer, autoincrement=True, primary_key=True)

    user_name = Column(Unicode(16), unique=True)

    email_address = Column(Unicode(255), unique=True)

    display_name = Column(Unicode(255))

    _password = Column('password', Unicode(40))

    created = Column(DateTime, default=datetime.now)

    def __repr__(self):
        return '<User: email="%s", display name="%s">' % (
                self.email_address, self.display_name)

    @property
    def permissions(self):
        perms = set()
        for g in self.groups:
            perms = perms | set(g.permissions)
        return perms

    @classmethod
    def by_email_address(cls, email):
        """A class method that can be used to search users
        based on their email addresses since it is unique.
        """
        return DBSession.query(cls).filter(cls.email_address==email).first()

    @classmethod
    def by_user_name(cls, username):
        """A class method that permits to search users
        based on their user_name attribute.
        """
        return DBSession.query(cls).filter(cls.user_name==username).first()


    def _set_password(self, password):
        """encrypts password on the fly using the encryption
        algo defined in the configuration
        """
        algorithm = config.get('authorize.hashmethod', None)
        self._password = self.__encrypt_password(algorithm, password)

    def _get_password(self):
        """returns password
        """
        return self._password

    password = synonym('password', descriptor=property(_get_password,
                                                       _set_password))

    def __encrypt_password(self, algorithm, password):
        """Hash the given password with the specified algorithm. Valid values
        for algorithm are 'md5' and 'sha1'. All other algorithm values will
        be essentially a no-op."""
        hashed_password = password

        if isinstance(password, unicode):
            password_8bit = password.encode('UTF-8')

        else:
            password_8bit = password

        if "md5" == algorithm:
            hashed_password = md5.new(password_8bit).hexdigest()

        elif "sha1" == algorithm:
            hashed_password = sha.new(password_8bit).hexdigest()

        # TODO: re-add the possibility to provide own hasing algo
        # here... just get the real config...

        #elif "custom" == algorithm:
        #    custom_encryption_path = turbogears.config.get(
        #        "identity.custom_encryption", None )
        #
        #    if custom_encryption_path:
        #        custom_encryption = turbogears.util.load_class(
        #            custom_encryption_path)

        #    if custom_encryption:
        #        hashed_password = custom_encryption(password_8bit)

        # make sure the hased password is an UTF-8 object at the end of the
        # process because SQLAlchemy _wants_ a unicode object for Unicode columns
        if not isinstance(hashed_password, unicode):
            hashed_password = hashed_password.decode('UTF-8')

        return hashed_password

    def validate_password(self, password):
        """Check the password against existing credentials.
        """
        identity = config.get('identity', None)
        if identity is None:
            return password
        algorithm = identity.get('password_encryption_method', None)
        return self.password == self.__encrypt_password(algorithm, password)


class Permission(DeclarativeBase):
    """A relationship that determines what each Group can do
    """
    __tablename__ = 'tg_permission'

    permission_id = Column(Integer, autoincrement=True, primary_key=True)

    permission_name = Column(Unicode(16), unique=True)

    description = Column(Unicode(255))

    groups = relation(Group, secondary=group_permission_table,
                      backref='permissions')



# -*- coding: utf-8 -*-

from ldapauth.tests import TestController

# This is an example of how you can write functional tests for your controller.
# As opposed to a pure unit-test which test a small unit of functionallity,
# these functional tests exercise the whole app and it's WSGI stack.
# Please read http://pythonpaste.org/webtest/ for more information

class TestRootController(TestController):
    def test_index(self):
        response = self.app.get('/')
        msg = 'TurboGears 2 is rapid web application development toolkit '\
              'designed to make your life easier.'
        # You can look for specific strings:
        self.failUnless(msg in response)
        # You cam also access a BeautifulSoup'ed version
        links = response.html.findAll('a')
        self.failUnless(links, "Mummy, there are no links here!")



# -*- coding: utf-8 -*-
#
# repoze.who.plugins.ldap, LDAP authentication for WSGI applications.
# Copyright (C) 2008 by Gustavo Narea <http://gustavonarea.net/>
#
# This file is part of repoze.who.plugins.ldap
# <http://code.gustavonarea.net/repoze.who.plugins.ldap/>
#
# repoze.who.plugins.ldap is freedomware: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by the
# Free Software Foundation, either version 3 of the License, or any later
# version.
#
# repoze.who.plugins.ldap is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of 
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# repoze.who.plugins.ldap. If not, see <http://www.gnu.org/licenses/>.

"""LDAP plugins for repoze.who."""

__all__ = ['LDAPAuthenticatorPlugin', 'LDAPAttributesPlugin']

from zope.interface import implements
import ldap

from repoze.who.interfaces import IAuthenticator, IMetadataProvider


#{ Authenticators


class LDAPAuthenticatorPlugin(object):

    implements(IAuthenticator)

    def __init__(self, ldap_connection, base_dn):
        """Create an LDAP authentication plugin.
        
        By passing an existing LDAPObject, you're free to use the LDAP
        authentication method you want, the way you want.
    
        If the default way to find the DN is not suitable for you, you may want
        to override L{_get_dn}.
        
        This plugin is compatible with any identifier plugin that defines the
        C{login} and C{password} items in the I{identity} dictionary.
        
        @param ldap_connection: An initialized LDAP connection.
        @type ldap_connection: C{ldap.ldapobject.SimpleLDAPObject}
        @param base_dn: The base for the I{Distinguished Name}. Something like
            C{ou=employees,dc=example,dc=org}, to which will be prepended the
            user id: C{uid=jsmith,ou=employees,dc=example,dc=org}.
        @type base_dn: C{unicode}
        @raise ValueError: If at least one of the parameters is not defined.
        
        """
        if base_dn is None:
            raise ValueError('A base Distinguished Name must be specified')
        self.ldap_connection = make_ldap_connection(ldap_connection)
        self.base_dn = base_dn

    # IAuthenticatorPlugin
    def authenticate(self, environ, identity):
        """Return the Distinguished Name of the user to be authenticated.
        
        @attention: The uid is not returned because it may not be unique; the
            DN, on the contrary, is always unique.
        @return: The Distinguished Name (DN), if the credentials were valid.
        @rtype: C{unicode} or C{None}
        
        """
        
        try:
            dn = self._get_dn(environ, identity)
            password = identity['password']
        except (KeyError, TypeError, ValueError):
            return None

        if not hasattr(self.ldap_connection, 'simple_bind_s'):
            environ['repoze.who.logger'].warn('Cannot bind with the provided '
                                              'LDAP connection object')
            return None
        
        try:
            self.ldap_connection.simple_bind_s(dn, password)
            # The credentials are valid!
            return dn
        except ldap.LDAPError:
            return None
    
    def _get_dn(self, environ, identity):
        """
        Return the DN based on the environment and the identity.
        
        It prepends the user id to the base DN given in the constructor:
        
        If the C{login} item of the identity is C{rms} and the base DN is
        C{ou=developers,dc=gnu,dc=org}, the resulting DN will be:
        C{uid=rms,ou=developers,dc=gnu,dc=org}.
        
        @attention: You may want to override this method if the DN generated by
            default doesn't meet your requirements. If you do so, make sure to
            raise a C{ValueError} exception if the operation is not successful.
        @param environ: The WSGI environment.
        @param identity: The identity dictionary.
        @return: The Distinguished Name (DN)
        @rtype: C{unicode}
        @raise ValueError: If the C{login} key is not in the I{identity} dict.
        
        """
        try:
            return u'uid=%s,%s' % (identity['login'], self.base_dn)
        except (KeyError, TypeError):
            raise ValueError

    def __repr__(self):
        return '<%s %s>' % (self.__class__.__name__, id(self))


#{ Metadata providers


class LDAPAttributesPlugin(object):
    """Loads LDAP attributes of the authenticated user."""
    
    implements(IMetadataProvider)
    
    def __init__(self, ldap_connection, attributes=None,
                 filterstr='(objectClass=*)'):
        """
        Fetch LDAP attributes of the authenticated user.
        
        @param ldap_connection: The LDAP connection to use to fetch this data.
        @type ldap_connection: C{ldap.ldapobject.SimpleLDAPObject} or C{str}
        @param attributes: The authenticated user's LDAP attributes you want to
            use in your application; an interable or a comma-separate list of
            attributes in a string, or C{None} to fetch them all.
        @type attributes: C{iterable} or C{str}
        @param filterstr: A filter for the search, as documented in U{RFC4515
            <http://www.faqs.org/rfcs/rfc4515.html>}; the results won't be
            filtered unless you define this.
        @type filterstr: C{str}
        @raise ValueError: If L{make_ldap_connection} could not create a
            connection from C{ldap_connection}, or if C{attributes} is not an
            iterable.
        
        """
        if hasattr(attributes, 'split'):
            attributes = attributes.split(',')
        elif hasattr(attributes, '__iter__'):
            # Converted to list, just in case...
            attributes = list(attributes)
        elif attributes is not None:
            raise ValueError('The needed LDAP attributes are not valid')
        self.ldap_connection = make_ldap_connection(ldap_connection)
        self.attributes = attributes
        self.filterstr = filterstr
    
    # IMetadataProvider
    def add_metadata(self, environ, identity):
        """
        Add metadata about the authenticated user to the identity.
        
        It modifies the C{identity} dictionary to add the metadata.
        
        @param environ: The WSGI environment.
        @param identity: The repoze.who's identity dictionary.
        
        """
        # Search arguments:
        args = (
            identity.get('repoze.who.userid'),
            ldap.SCOPE_BASE,
            self.filterstr,
            self.attributes
        )
        try:
            for (dn, attributes) in self.ldap_connection.search_s(*args):
                identity.update(attributes)
        except ldap.LDAPError, msg:
            environ['repoze.who.logger'].warn('Cannot add metadata: %s' % \
                                              msg)
            return


#{ Utilities


def make_ldap_connection(ldap_connection):
    """Return an LDAP connection object to the specified server.
    
    If the C{ldap_connection} is already an LDAP connection object, it will
    be returned as is. If it's an LDAP URL, it will return an LDAP connection
    to the LDAP server specified in the URL.
    
    @param ldap_connection: The LDAP connection object or the LDAP URL of the
        server to be connected to.
    @type ldap_connection: C{ldap.ldapobject.SimpleLDAPObject}, C{str} or
        C{unicode}
    @return: The LDAP connection object.
    @rtype: C{ldap.ldapobject.SimpleLDAPObject}
    @raise ValueError: If C{ldap_connection} is C{None}.
    
    """
    if isinstance(ldap_connection, str) or isinstance(ldap_connection, unicode):
        return ldap.initialize(ldap_connection)
    elif ldap_connection is None:
        raise ValueError('An LDAP connection must be specified')
    return ldap_connection


#}



"""Setup the LDAPAuth application"""
import logging

import transaction
from paste.deploy import appconfig
from tg import config

from ldapauth.config.environment import load_environment

log = logging.getLogger(__name__)

def setup_config(command, filename, section, vars):
    """Place any commands to setup ldapauth here"""
    conf = appconfig('config:' + filename)
    load_environment(conf.global_conf, conf.local_conf)
    # Load the models
    #from ldapauth import model
    #print "Creating tables"
    #model.metadata.create_all(bind=config['pylons.app_globals'].sa_engine)

    print """
================= Demo Project for repoze.who.plugins.ldap ====================

You should setup this demo by creating the relevant users and groups in your
LDAP server, and then adjusting the relevant parameters in the "who.ini" file.

"""
    
    #transaction.commit()
    
    print "Successfully setup"



from tg.configuration import AppConfig, Bunch
import ldapauth
from ldapauth import model
from ldapauth.lib import app_globals, helpers

base_config = AppConfig()
base_config.renderers = []

base_config.package = ldapauth

#Set the default renderer
base_config.default_renderer = 'genshi'
base_config.renderers.append('genshi') 

#Configure the base SQLALchemy Setup
base_config.use_sqlalchemy = True
base_config.model = ldapauth.model
base_config.DBSession = ldapauth.model.DBSession




from identification import OpenIdIdentificationPlugin

def make_identification_plugin(store='mem',
                openid_field = "openid",
                session_name = None,
                login_handler_path = None,
                logout_handler_path = None,
                login_form_url = None,
                error_field = 'error',
                logged_in_url = None,
                logged_out_url = None,
                came_from_field = 'came_from',
                store_file_path='',
                rememberer_name = None,
                sql_associations_table = '',
                sql_nonces_table = '',
                sql_connstring = ''):
    if store not in (u'mem',u'file',u'sql'):
        raise ValueError("store needs to be 'mem', 'sql' or 'file'")
    if login_form_url is None:
        raise ValueError("login_form_url needs to be given")
    if rememberer_name is None:
        raise ValueError("rememberer_name needs to be given")
    if login_handler_path is None:
        raise ValueError("login_handler_path needs to be given")
    if logout_handler_path is None:
        raise ValueError("logout_handler_path needs to be given")
    if session_name is None:
        raise ValueError("session_name needs to be given")
    if logged_in_url is None:
        raise ValueError("logged_in_url needs to be given")
    if logged_out_url is None:
        raise ValueError("logged_out_url needs to be given")

    plugin = OpenIdIdentificationPlugin(store, 
        openid_field = openid_field,
        error_field = error_field,
        session_name = session_name,
        login_form_url = login_form_url,
        login_handler_path = login_handler_path,
        logout_handler_path = logout_handler_path,
        store_file_path = store_file_path,
        logged_in_url = logged_in_url,
        logged_out_url = logged_out_url,
        came_from_field = came_from_field,
        rememberer_name = rememberer_name,
        sql_associations_table = sql_associations_table,
        sql_nonces_table = sql_nonces_table,
        sql_connstring = sql_connstring
        )
    return plugin




from setuptools import setup, find_packages
import os

version = '0.5'

setup(name='repoze.who.plugins.openid',
      version=version,
      description="An OpenID plugin for repoze.who",
      long_description=open("README.txt").read() + "\n" +
                       open(os.path.join("docs", "HISTORY.txt")).read(),
      # Get more strings from http://www.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        "Programming Language :: Python",
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU General Public License (GPL)",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Internet :: WWW/HTTP :: WSGI :: Middleware",
        "Topic :: System :: Systems Administration :: Authentication/Directory :: LDAP" 
        ],
      keywords='openid repoze who identification authentication plugin',
      author='Christian Scholz',
      author_email='cs@comlounge.net',
      url='http://quantumcore.org/docs/repoze.who.plugins.openid',
      license="BSD-derived (http://www.repoze.org/LICENSE.txt)",
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['repoze', 'repoze.who', 'repoze.who.plugins'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'repoze.who>=1.0.6', 
	  'python-openid>=2.0',
          'setuptools',
          'zope.interface'
      ],
      test_requires=[   
      ],
      test_suite="repoze.who.plugins.openid",
      entry_points="""
      # -*- Entry points: -*-
      """,
      test_suite='repoze.who.plugins.openid.tests'
      )



# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009, Gustavo Narea <me@gustavonarea.net>.
# All Rights Reserved.
#
# This software is subject to the provisions of the BSD-like license at
# http://www.repoze.org/LICENSE.txt.  A copy of the license should accompany
# this distribution.  THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL
# EXPRESS OR IMPLIED WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO,
# THE IMPLIED WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND
# FITNESS FOR A PARTICULAR PURPOSE.
#
##############################################################################

"""Test suite for the collection of :mod:`repoze.who` friendly forms."""

from unittest import TestCase
from urllib import quote as original_quoter

from zope.interface.verify import verifyClass
from paste.httpexceptions import HTTPFound
from repoze.who.interfaces import IIdentifier, IChallenger

from repoze.who.plugins.friendlyform import FriendlyFormPlugin

# Let's prevent the original quote() from leaving slashes:
quote = lambda txt: original_quoter(txt, '')


class TestFriendlyFormPlugin(TestCase):

    def test_implements(self):
        verifyClass(IIdentifier, FriendlyFormPlugin)
        verifyClass(IChallenger, FriendlyFormPlugin)
    
    def test_constructor(self):
        p = self._make_one()
        self.assertEqual(p.login_counter_name, '__logins')
        self.assertEqual(p.post_login_url, None)
        self.assertEqual(p.post_logout_url, None)
    
    def test_constructor_with_loging_counter_as_None(self):
        p = self._make_one(login_counter_name=None)
        self.assertEqual(p.login_counter_name, '__logins')
    
    def test_repr(self):
        p = self._make_one()
        self.assertEqual(repr(p), '<FriendlyFormPlugin %s>' % id(p))
    
    def test_login_without_postlogin_page(self):
        """
        The page to be redirected to after login must include the login 
        counter.
        
        """
        # --- Configuring the plugin:
        p = self._make_one()
        # --- Configuring the mock environ:
        came_from = '/some_path'
        environ = self._make_environ('/login_handler',
                                     'came_from=%s' % quote(came_from))
        # --- Testing it:
        p.identify(environ)
        app = environ['repoze.who.application']
        new_redirect = came_from + '?__logins=0'
        self.assertEqual(app.location(), new_redirect)
    
    def test_post_login_page_as_url(self):
        """Post-logout pages can also be defined as URLs, not only paths"""
        # --- Configuring the plugin:
        login_url = 'http://example.org/welcome'
        p = self._make_one(post_login_url=login_url)
        # --- Configuring the mock environ:
        environ = self._make_environ('/login_handler')
        # --- Testing it:
        p.identify(environ)
        app = environ['repoze.who.application']
        self.assertEqual(app.location(), login_url + '?__logins=0')
    
    def test_post_login_page_with_SCRIPT_NAME(self):
        """
        While redirecting to the post-login page, the SCRIPT_NAME must be
        taken into account.
        
        """
        # --- Configuring the plugin:
        p = self._make_one(post_login_url='/welcome_back')
        # --- Configuring the mock environ:
        environ = self._make_environ('/login_handler', SCRIPT_NAME='/my-app')
        # --- Testing it:
        p.identify(environ)
        app = environ['repoze.who.application']
        self.assertEqual(app.location(), '/my-app/welcome_back?__logins=0')
    
    def test_post_login_page_with_SCRIPT_NAME_and_came_from(self):
        """
        While redirecting to the post-login page with the came_from variable, 
        the SCRIPT_NAME must be taken into account.
        
        """
        # --- Configuring the plugin:
        p = self._make_one(post_login_url='/welcome_back')
        # --- Configuring the mock environ:
        came_from = '/something'
        environ = self._make_environ('/login_handler',
                                     'came_from=%s' % quote(came_from),
                                     SCRIPT_NAME='/my-app')
        # --- Testing it:
        p.identify(environ)
        app = environ['repoze.who.application']
        redirect = '/my-app/welcome_back?__logins=0&came_from=%s'
        self.assertEqual(app.location(), redirect % quote(came_from))
    
    def test_post_login_page_without_login_counter(self):
        """
        If there's no login counter defined, the post-login page should receive
        the counter at zero.
        
        """
        # --- Configuring the plugin:
        p = self._make_one(post_login_url='/welcome_back')
        # --- Configuring the mock environ:
        environ = self._make_environ('/login_handler')
        # --- Testing it:
        p.identify(environ)
        app = environ['repoze.who.application']
        self.assertEqual(app.location(), '/welcome_back?__logins=0')
    
    def test_post_login_page_with_login_counter(self):
        """
        If the login counter is defined, the post-login page should receive it
        as is.
        
        """
        # --- Configuring the plugin:
        p = self._make_one(post_login_url='/welcome_back')
        # --- Configuring the mock environ:
        environ = self._make_environ('/login_handler', '__logins=2',
                                     redirect='/some_path')
        # --- Testing it:
        p.identify(environ)
        app = environ['repoze.who.application']
        self.assertEqual(app.location(), '/welcome_back?__logins=2')
    
    def test_post_login_page_with_invalid_login_counter(self):
        """
        If the login counter is defined with an invalid value, the post-login 
        page should receive the counter at zero.
        
        """
        # --- Configuring the plugin:
        p = self._make_one(post_login_url='/welcome_back')
        # --- Configuring the mock environ:
        environ = self._make_environ('/login_handler', '__logins=non_integer',
                                     redirect='/some_path')
        # --- Testing it:
        p.identify(environ)
        app = environ['repoze.who.application']
        self.assertEqual(app.location(), '/welcome_back?__logins=0')
    
    def test_post_login_page_with_referrer(self):
        """
        If the referrer is defined, it should be passed along with the login
        counter to the post-login page.
        
        """
        # --- Configuring the plugin:
        p = self._make_one(post_login_url='/welcome_back')
        # --- Configuring the mock environ:
        orig_redirect = '/some_path'
        came_from = quote('http://example.org')
        environ = self._make_environ(
            '/login_handler',
            '__logins=3&came_from=%s' % came_from,
            redirect=orig_redirect,
            )
        # --- Testing it:
        p.identify(environ)
        app = environ['repoze.who.application']
        new_url = '/welcome_back?__logins=3&came_from=%s' % came_from
        self.assertEqual(app.location(), new_url)
    
    def test_login_page_with_login_counter(self):
        """
        In the page where the login form is displayed, the login counter
        must be defined in the WSGI environment variable 'repoze.who.logins'.
        
        """
        # --- Configuring the plugin:
        p = self._make_one()
        # --- Configuring the mock environ:
        environ = self._make_environ('/login', '__logins=2')
        # --- Testing it:
        p.identify(environ)
        self.assertEqual(environ['repoze.who.logins'], 2)
        self.assertEqual(environ['QUERY_STRING'], '')
    
    def test_login_page_without_login_counter(self):
        """
        In the page where the login form is displayed, the login counter
        must be defined in the WSGI environment variable 'repoze.who.logins' 
        and if it's not defined in the query string, set it to zero in the
        environ.
        
        """
        # --- Configuring the plugin:
        p = self._make_one()
        # --- Configuring the mock environ:
        environ = self._make_environ('/login')
        # --- Testing it:
        p.identify(environ)
        self.assertEqual(environ['repoze.who.logins'], 0)
        self.assertEqual(environ['QUERY_STRING'], '')
    
    def test_login_page_with_camefrom(self):
        """
        In the page where the login form is displayed, the login counter
        must be defined in the WSGI environment variable 'repoze.who.logins' 
        and hidden in the query string available in the environ.
        
        """
        # --- Configuring the plugin:
        p = self._make_one()
        # --- Configuring the mock environ:
        came_from = 'http://example.com'
        environ = self._make_environ('/login',
                                     'came_from=%s' % quote(came_from))
        # --- Testing it:
        p.identify(environ)
        self.assertEqual(environ['repoze.who.logins'], 0)
        self.assertEqual(environ['QUERY_STRING'], 
                         'came_from=%s' % quote(came_from))
    
    def test_logout_without_post_logout_page(self):
        """
        Users must be redirected to '/' on logout if there's no referrer page
        and no post-logout page defined.
        
        """
        # --- Configuring the plugin:
        p = self._make_one()
        # --- Configuring the mock environ:
        environ = self._make_environ('/logout_handler')
        # --- Testing it:
        app = p.challenge(environ, '401 Unauthorized', [('app', '1')],
                          [('forget', '1')])
        self.assertEqual(app.location(), '/')
    
    def test_logout_with_SCRIPT_NAME_and_without_post_logout_page(self):
        """
        Users must be redirected to SCRIPT_NAME on logout if there's no 
        referrer page and no post-logout page defined.
        
        """
        # --- Configuring the plugin:
        p = self._make_one()
        # --- Configuring the mock environ:
        environ = self._make_environ('/logout_handler', SCRIPT_NAME='/my-app')
        # --- Testing it:
        app = p.challenge(environ, '401 Unauthorized', [('app', '1')],
                          [('forget', '1')])
        self.assertEqual(app.location(), '/my-app')
    
    def test_logout_with_camefrom_and_without_post_logout_page(self):
        """
        Users must be redirected to the referrer page on logout if there's no
        post-logout page defined.
        
        """
        # --- Configuring the plugin:
        p = self._make_one()
        # --- Configuring the mock environ:
        environ = self._make_environ('/logout_handler')
        environ['came_from'] = '/somewhere'
        # --- Testing it:
        app = p.challenge(environ, '401 Unauthorized', [('app', '1')],
                          [('forget', '1')])
        self.assertEqual(app.location(), '/somewhere')
    
    def test_logout_with_post_logout_page(self):
        """Users must be redirected to the post-logout page, if defined"""
        # --- Configuring the plugin:
        p = self._make_one(post_logout_url='/see_you_later')
        # --- Configuring the mock environ:
        environ = self._make_environ('/logout_handler')
        # --- Testing it:
        app = p.challenge(environ, '401 Unauthorized', [('app', '1')],
                          [('forget', '1')])
        self.assertEqual(app.location(), '/see_you_later')
    
    def test_logout_with_post_logout_page_as_url(self):
        """Post-logout pages can also be defined as URLs, not only paths"""
        # --- Configuring the plugin:
        logout_url = 'http://example.org/see_you_later'
        p = self._make_one(post_logout_url=logout_url)
        # --- Configuring the mock environ:
        environ = self._make_environ('/logout_handler')
        # --- Testing it:
        app = p.challenge(environ, '401 Unauthorized', [('app', '1')],
                          [('forget', '1')])
        self.assertEqual(app.location(), logout_url)
    
    def test_logout_with_post_logout_page_and_SCRIPT_NAME(self):
        """
        Users must be redirected to the post-logout page, if defined, taking
        the SCRIPT_NAME into account.
        
        """
        # --- Configuring the plugin:
        p = self._make_one(post_logout_url='/see_you_later')
        # --- Configuring the mock environ:
        environ = self._make_environ('/logout_handler', SCRIPT_NAME='/my-app')
        # --- Testing it:
        app = p.challenge(environ, '401 Unauthorized', [('app', '1')],
                          [('forget', '1')])
        self.assertEqual(app.location(), '/my-app/see_you_later')
    
    def test_logout_with_post_logout_page_and_came_from(self):
        """
        Users must be redirected to the post-logout page, if defined, and also
        pass the came_from variable.
        
        """
        # --- Configuring the plugin:
        p = self._make_one(post_logout_url='/see_you_later')
        # --- Configuring the mock environ:
        came_from = '/the-path'
        environ = self._make_environ('/logout_handler')
        environ['came_from'] = came_from
        # --- Testing it:
        app = p.challenge(environ, '401 Unauthorized', [('app', '1')],
                          [('forget', '1')])
        redirect = '/see_you_later?came_from=%s'
        self.assertEqual(app.location(), redirect % quote(came_from))
    
    def test_failed_login(self):
        """
        Users must be redirected to the login form if the tried to log in with
        the wrong credentials.
        
        """
        # --- Configuring the plugin:
        p = self._make_one()
        # --- Configuring the mock environ:
        environ = self._make_environ('/somewhere')
        environ['repoze.who.logins'] = 1
        # --- Testing it:
        app = p.challenge(environ, '401 Unauthorized', [('app', '1')],
                          [('forget', '1')])
        came_from = 'http://example.org/somewhere'
        redirect = '/login?__logins=2&came_from=%s' % quote(came_from)
        self.assertEqual(app.location(), redirect)
    
    def test_not_logout_and_not_failed_logins(self):
        """
        Do not modify the challenger unless it's handling a logout or a
        failed login.
        
        """
        # --- Configuring the plugin:
        p = self._make_one()
        # --- Configuring the mock environ:
        environ = self._make_environ('/somewhere')
        # --- Testing it:
        app = p.challenge(environ, '401 Unauthorized', [('app', '1')],
                          [('forget', '1')])
        came_from = 'http://example.org/somewhere'
        redirect = '/login?came_from=%s' % quote(came_from)
        self.assertEqual(app.location(), redirect)

    def test_identify_pathinfo_miss(self):
        plugin = self._makeOne()
        environ = self._makeFormEnviron(path_info='/not_login_handler')
        result = plugin.identify(environ)
        self.assertEqual(result, None)
        self.failIf(environ.get('repoze.who.application'))

    def test_identify_via_login_handler(self):
        plugin = self._makeOne()
        environ = self._makeFormEnviron(path_info='/login_handler',
                                        login='chris',
                                        password='password',
                                        came_from='http://example.com/')
        result = plugin.identify(environ)
        self.assertEqual(result, {'login':'chris', 'password':'password'})
        app = environ['repoze.who.application']
        self.assertEqual(len(app.headers), 1)
        name, value = app.headers[0]
        self.assertEqual(name, 'location')
        self.assertEqual(value, 'http://example.com/?__logins=0')
        self.assertEqual(app.code, 302)

    def test_identify_via_login_handler_no_username_pass(self):
        plugin = self._makeOne()
        environ = self._makeFormEnviron(path_info='/login_handler')
        result = plugin.identify(environ)
        self.assertEqual(result, None)
        app = environ['repoze.who.application']
        self.assertEqual(len(app.headers), 1)
        name, value = app.headers[0]
        self.assertEqual(name, 'location')
        self.assertEqual(value, '/?__logins=0')
        self.assertEqual(app.code, 302)

    def test_identify_via_login_handler_no_came_from_no_http_referer(self):
        plugin = self._makeOne()
        environ = self._makeFormEnviron(path_info='/login_handler',
                                        login='chris',
                                        password='password')
        result = plugin.identify(environ)
        self.assertEqual(result, {'login':'chris', 'password':'password'})
        app = environ['repoze.who.application']
        self.assertEqual(len(app.headers), 1)
        name, value = app.headers[0]
        self.assertEqual(name, 'location')
        self.assertEqual(value, '/?__logins=0')
        self.assertEqual(app.code, 302)

    def test_identify_via_login_handler_no_came_from(self):
        plugin = self._makeOne()
        environ = self._makeFormEnviron(path_info='/login_handler',
                                        login='chris',
                                        password='password')
        environ['HTTP_REFERER'] = 'http://foo.bar/'
        result = plugin.identify(environ)
        self.assertEqual(result, {'login':'chris', 'password':'password'})
        app = environ['repoze.who.application']
        self.assertEqual(len(app.headers), 1)
        name, value = app.headers[0]
        self.assertEqual(name, 'location')
        self.assertEqual(value, 'http://foo.bar/?__logins=0')
        self.assertEqual(app.code, 302)

    def test_identify_via_login_handler_no_came_from_no_referer_sname(self):
        plugin = self._makeOne()
        environ = self._makeFormEnviron(path_info='/login_handler',
                                        script_name='/my-app',
                                        login='chris',
                                        password='password')
        plugin.identify(environ)
        app = environ['repoze.who.application']
        self.assertEqual(app.location(), '/my-app?__logins=0')

    def test_identify_via_logout_handler(self):
        plugin = self._makeOne()
        environ = self._makeFormEnviron(path_info='/logout_handler',
                                        login='chris',
                                        password='password',
                                        came_from='http://example.com')
        result = plugin.identify(environ)
        self.assertEqual(result, None)
        app = environ['repoze.who.application']
        self.assertEqual(len(app.headers), 0)
        self.assertEqual(app.code, 401)
        self.assertEqual(environ['came_from'], 'http://example.com')

    def test_identify_via_logout_handler_no_came_from_no_http_referer(self):
        plugin = self._makeOne()
        environ = self._makeFormEnviron(path_info='/logout_handler',
                                        login='chris',
                                        password='password')
        result = plugin.identify(environ)
        self.assertEqual(result, None)
        app = environ['repoze.who.application']
        self.assertEqual(len(app.headers), 0)
        self.assertEqual(app.code, 401)
        self.assertEqual(environ['came_from'], '/')

    def test_identify_via_logout_handler_no_came_from_no_referer_spath(self):
        plugin = self._makeOne()
        environ = self._makeFormEnviron(path_info='/logout_handler',
                                        script_name='/my-app',
                                        login='chris',
                                        password='password')
        plugin.identify(environ)
        app = environ['repoze.who.application']
        self.assertEqual(environ['came_from'], '/my-app')

    def test_identify_via_logout_handler_no_came_from(self):
        plugin = self._makeOne()
        environ = self._makeFormEnviron(path_info='/logout_handler',
                                        login='chris',
                                        password='password')
        environ['HTTP_REFERER'] = 'http://example.com/referer'
        result = plugin.identify(environ)
        self.assertEqual(result, None)
        app = environ['repoze.who.application']
        self.assertEqual(len(app.headers), 0)
        self.assertEqual(app.code, 401)
        self.assertEqual(environ['came_from'], 'http://example.com/referer')

    def test_remember(self):
        plugin = self._makeOne()
        environ = self._makeFormEnviron()
        identity = {}
        result = plugin.remember(environ, identity)
        self.assertEqual(result, None)
        self.assertEqual(environ['repoze.who.plugins']['cookie'].remembered,
                         identity)

    def test_forget(self):
        plugin = self._makeOne()
        environ = self._makeFormEnviron()
        identity = {}
        result = plugin.forget(environ, identity)
        self.assertEqual(result, None)
        self.assertEqual(environ['repoze.who.plugins']['cookie'].forgotten,
                         identity
                         )

    def test_challenge(self):
        plugin = self._makeOne()
        environ = self._makeFormEnviron()
        app = plugin.challenge(environ, '401 Unauthorized', [('app', '1')],
                               [('forget', '1')])
        sr = DummyStartResponse()
        result = ''.join(app(environ, sr))
        self.failUnless(result.startswith('302 Found'))
        self.assertEqual(len(sr.headers), 3)
        self.assertEqual(sr.headers[1][0], 'location')
        url = sr.headers[1][1]
        import urlparse
        import cgi
        parts = urlparse.urlparse(url)
        parts_qsl = cgi.parse_qsl(parts[4])
        self.assertEqual(len(parts_qsl), 1)
        came_from_key, came_from_value = parts_qsl[0]
        self.assertEqual(parts[0], 'http')
        self.assertEqual(parts[1], 'example.com')
        self.assertEqual(parts[2], '/login.html')
        self.assertEqual(parts[3], '')
        self.assertEqual(came_from_key, 'came_from')
        self.assertEqual(came_from_value, 'http://www.example.com/?default=1')
        headers = sr.headers
        self.assertEqual(len(headers), 3)
        self.assertEqual(sr.headers[0][0], 'forget')
        self.assertEqual(sr.headers[0][1], '1')
        self.assertEqual(sr.headers[2][0], 'content-type')
        self.assertEqual(sr.headers[2][1], 'text/plain; charset=utf8')
        self.assertEqual(sr.status, '302 Found')

    def test_challenge_came_from_in_environ(self):
        plugin = self._makeOne()
        environ = self._makeFormEnviron()
        environ['came_from'] = 'http://example.com/came_from'
        app = plugin.challenge(environ, '401 Unauthorized', [('app', '1')],
                               [('forget', '1')])
        sr = DummyStartResponse()
        result = ''.join(app(environ, sr))
        self.failUnless(result.startswith('302 Found'))
        self.assertEqual(len(sr.headers), 3)
        self.assertEqual(sr.headers[1][0], 'location')
        url = sr.headers[1][1]
        import urlparse
        import cgi
        parts = urlparse.urlparse(url)
        parts_qsl = cgi.parse_qsl(parts[4])
        self.assertEqual(len(parts_qsl), 1)
        came_from_key, came_from_value = parts_qsl[0]
        self.assertEqual(parts[0], 'http')
        self.assertEqual(parts[1], 'example.com')
        self.assertEqual(parts[2], '/login.html')
        self.assertEqual(parts[3], '')
        self.assertEqual(came_from_key, 'came_from')
        self.assertEqual(came_from_value, 'http://example.com/came_from')

    def test_challenge_with_setcookie_from_app(self):
        plugin = self._makeOne()
        environ = self._makeFormEnviron()
        app = plugin.challenge(
            environ,
            '401 Unauthorized',
            [('app', '1'), ('set-cookie','a'), ('set-cookie','b')],
            [])
        sr = DummyStartResponse()
        result = ''.join(app(environ, sr))
        self.failUnless(result.startswith('302 Found'))
        self.assertEqual(sr.headers[0][0], 'set-cookie')
        self.assertEqual(sr.headers[0][1], 'a')
        self.assertEqual(sr.headers[1][0], 'set-cookie')
        self.assertEqual(sr.headers[1][1], 'b')
        self.assertEqual(sr.headers[2][0], 'location')

    def test_challenge_with_non_root_script_name(self):
        """The script name must be taken into account while redirecting."""
        plugin = self._makeOne(login_form_url='/login')
        environ = self._makeFormEnviron(script_name='/app',
                                        path_info='/admin')
        came_from = 'http://www.example.com/app/admin?default=1'
        environ['came_from'] = came_from
        app = plugin.challenge(environ, '401 Unauthorized', [('app', '1')],
                               [('forget', '1')])
        from urllib import quote
        login_url = '/app/login?came_from=%s' % quote(came_from, '')
        self.assertEqual(app.location(), login_url)
    
    def _make_one(self, login_counter_name='__logins', post_login_url=None,
                  post_logout_url=None):
        p = FriendlyFormPlugin('/login', '/login_handler', post_login_url,
                               '/logout_handler', post_logout_url,
                               'whatever',
                               login_counter_name=login_counter_name)
        return p

    def _makeOne(self, login_form_url='http://example.com/login.html',
                 login_handler_path = '/login_handler',
                 logout_handler_path = '/logout_handler',
                 rememberer_name='cookie'):
        # TODO: Merge this into _make_one()
        plugin = FriendlyFormPlugin(login_form_url, login_handler_path, None,
                                    logout_handler_path, None, rememberer_name)
        return plugin
    
    def _make_redirection(self, url):
        # TODO: Remove this method
        app = HTTPFound(url)
        return app
    
    def _make_environ(self, path_info, qs='', SCRIPT_NAME='', redirect=None):
        environ = {
            'PATH_INFO': path_info,
            'SCRIPT_NAME': SCRIPT_NAME,
            'QUERY_STRING': qs,
            'SERVER_NAME': 'example.org',
            'SERVER_PORT': '80',
            'wsgi.input': '',
            'wsgi.url_scheme': 'http',
            }
        # TODO: Remove the ``redirect`` param
        if redirect:
            environ['repoze.who.application'] = self._make_redirection(redirect)
        return environ
    
    def _makeEnviron(self, kw=None):
        # TODO: Merge this into _make_environ
        environ = {}
        environ['wsgi.version'] = (1,0)
        if kw is not None:
            environ.update(kw)
        return environ

    def _makeFormEnviron(self, login=None, password=None, came_from=None,
                         path_info='/', identifier=None, script_name=''):
        # TODO: Merge this into _make_environ
        from StringIO import StringIO
        fields = []
        if login:
            fields.append(('login', login))
        if password:
            fields.append(('password', password))
        if came_from:
            fields.append(('came_from', came_from))
        if identifier is None:
            credentials = {'login':'chris', 'password':'password'}
            identifier = DummyIdentifier(credentials)
        content_type, body = encode_multipart_formdata(fields)
        extra = {'wsgi.input':StringIO(body),
                 'wsgi.url_scheme':'http',
                 'SERVER_NAME':'www.example.com',
                 'SERVER_PORT':'80',
                 'CONTENT_TYPE':content_type,
                 'CONTENT_LENGTH':len(body),
                 'REQUEST_METHOD':'POST',
                 'repoze.who.plugins': {'cookie':identifier},
                 'QUERY_STRING':'default=1',
                 'PATH_INFO':path_info,
                 'SCRIPT_NAME':script_name
                 }
        environ = self._makeEnviron(extra)
        return environ


#{ Utilities


def encode_multipart_formdata(fields):
    BOUNDARY = '----------ThIs_Is_tHe_bouNdaRY_$'
    CRLF = '\r\n'
    L = []
    for (key, value) in fields:
        L.append('--' + BOUNDARY)
        L.append('Content-Disposition: form-data; name="%s"' % key)
        L.append('')
        L.append(value)
    L.append('--' + BOUNDARY + '--')
    L.append('')
    body = CRLF.join(L)
    content_type = 'multipart/form-data; boundary=%s' % BOUNDARY
    return content_type, body


#{ Mock objects


class DummyStartResponse:
    def __call__(self, status, headers, exc_info=None):
        self.status = status
        self.headers = headers
        self.exc_info = exc_info
        return []


class DummyIdentifier:
    forgotten = False
    remembered = False

    def __init__(self, credentials=None, remember_headers=None,
                 forget_headers=None, replace_app=None):
        self.credentials = credentials
        self.remember_headers = remember_headers
        self.forget_headers = forget_headers
        self.replace_app = replace_app

    def identify(self, environ):
        if self.replace_app:
            environ['repoze.who.application'] = self.replace_app
        return self.credentials

    def forget(self, environ, identity):
        self.forgotten = identity
        return self.forget_headers

    def remember(self, environ, identity):
        self.remembered = identity
        return self.remember_headers

#}



import zope.interface 
from repoze.who.interfaces import IChallengeDecider


def openid_challenge_decider(environ, status, headers):
    # we do the default if it's a 401, probably we show a form then
    if status.startswith('401 '):
        return True
    elif environ.has_key('repoze.whoplugins.openid.openid'):
        # in case IIdentification found an openid it should be in the environ
        # and we do the challenge
        return True
    return False
    
zope.interface.directlyProvides(openid_challenge_decider, IChallengeDecider)




import cgi
import urlparse
import cgitb
import sys
from zope.interface import implements

from repoze.who.interfaces import IChallenger
from repoze.who.interfaces import IIdentifier
from repoze.who.interfaces import IAuthenticator

from webob import Request, Response

import openid
from openid.store import memstore, filestore, sqlstore
from openid.consumer import consumer
from openid.oidutil import appendArgs
from openid.cryptutil import randomString
from openid.fetchers import setDefaultFetcher, Urllib2Fetcher
from openid.extensions import pape, sreg



class OpenIdIdentificationPlugin(object):
    """The repoze.who OpenID plugin
    
    This class contains 3 plugin types and is thus implementing
    IIdentifier, IChallenger and IAuthenticator.
    (check the `repoze.who documentation <http://static.repoze.org/bfgdocs/>`_
    for what all these plugin types do.)
    
    """


    implements(IChallenger, IIdentifier, IAuthenticator)

    def __init__(self, store, 
                    openid_field, 
                    error_field = '',
                    store_file_path='',
                    session_name = '',
                    login_handler_path = '',
                    logout_handler_path = '',
                    login_form_url = '',
                    logged_in_url = '',
                    logged_out_url = '',
                    came_from_field = '',
                    rememberer_name = '',
                    sql_associations_table = '',
                    sql_nonces_table = '',
                    sql_connstring = ''):

        self.rememberer_name = rememberer_name
        self.login_handler_path = login_handler_path
        self.logout_handler_path = logout_handler_path
        self.login_form_url = login_form_url
        self.session_name = session_name
        self.error_field = error_field
        self.came_from_field = came_from_field
        self.logged_out_url = logged_out_url
        self.logged_in_url = logged_in_url
        
        # for the SQL store
        self.sql_associations_table = sql_associations_table
        self.sql_nonces_table = sql_nonces_table
        self.sql_connstring = sql_connstring
        
        # set up the store
        if store==u"file":
            self.store = filestore.FileOpenIDStore(store_file_path)
        elif store==u"mem":
            self.store = memstore.MemoryStore()
        elif store==u"sql":
            # TODO: This does not work as we need a connection, not a string
            self.store = sqlstore.SQLStore(sql_connstring, sql_associations_table, sql_connstring)
        self.openid_field = openid_field
            
    def _get_rememberer(self, environ):
        rememberer = environ['repoze.who.plugins'][self.rememberer_name]
        return rememberer

    def get_consumer(self,environ):
        session = environ.get(self.session_name,{})
        return consumer.Consumer(session,self.store)
        
    def redirect_to_logged_in(self, environ):
        """redirect to came_from or standard page if login was successful"""
        request = Request(environ)
        came_from = request.params.get(self.came_from_field,'')
        if came_from!='':
            url = came_from
        else:
            url = self.logged_in_url
        res = Response()
        res.status = 302
        res.location = url
        environ['repoze.who.application'] = res    

    # IIdentifier
    def identify(self, environ):
        """this method is called when a request is incoming.

        After the challenge has been called we might get here a response
        from an openid provider.

        """

        request = Request(environ)
        
        # first test for logout as we then don't need the rest
        if request.path == self.logout_handler_path:
            res = Response()
            # set forget headers
            for a,v in self.forget(environ,{}):
                res.headers.add(a,v)
            res.status = 302
            res.location = self.logged_out_url
            environ['repoze.who.application'] = res
            return {}

        identity = {}

        # first we check we are actually on the URL which is supposed to be the
        # url to return to (login_handler_path in configuration)
        # this URL is used for both: the answer for the login form and
        # when the openid provider redirects the user back.
        if request.path == self.login_handler_path:

        # in the case we are coming from the login form we should have 
        # an openid in here the user entered
            open_id = request.params.get(self.openid_field, None)
            environ['repoze.who.logger'].debug('checking openid results for : %s ' %open_id)
            
            if open_id is not None:
                open_id = open_id.strip()
            
            # we don't do anything with the openid we found ourselves but we put it in here
            # to tell the challenge plugin to initiate the challenge
            identity['repoze.whoplugins.openid.openid'] = environ['repoze.whoplugins.openid.openid'] = open_id
        
            # this part now is for the case when the openid provider redirects
            # the user back. We should find some openid specific fields in the request.
            mode=request.params.get("openid.mode", None)
            if mode=="id_res":
                oidconsumer = self.get_consumer(environ)
                info = oidconsumer.complete(request.params, request.url)

                if info.status == consumer.SUCCESS:
                    environ['repoze.who.logger'].info('openid request successful for : %s ' %open_id)
                    
                    display_identifier = info.identity_url
                    
                    # remove this so that the challenger is not triggered again
                    del environ['repoze.whoplugins.openid.openid']
                    
                    # store the id for the authenticator
                    identity['repoze.who.plugins.openid.userid'] = display_identifier

                    # now redirect to came_from or the success page
                    self.redirect_to_logged_in(environ)
                    return identity
                    
                # TODO: Do we have to check for more failures and such?
                # 
            elif mode=="cancel":
                # cancel is a negative assertion in the OpenID protocol,
                # which means the user did not authorize correctly.
                environ['repoze.whoplugins.openid.error'] = 'OpenID authentication failed.'
                pass
        return identity

    # IIdentifier
    def remember(self, environ, identity):
        """remember the openid in the session we have anyway"""
        rememberer = self._get_rememberer(environ)
        r = rememberer.remember(environ, identity)
        return r

    # IIdentifier
    def forget(self, environ, identity):
        """forget about the authentication again"""
        rememberer = self._get_rememberer(environ)
        return rememberer.forget(environ, identity)

    # IChallenge
    def challenge(self, environ, status, app_headers, forget_headers):
        """the challenge method is called when the ``IChallengeDecider``
        in ``classifiers.py`` returns ``True``. This is the case for either a 
        ``401`` response from the client or if the key 
        ``repoze.whoplugins.openid.openidrepoze.whoplugins.openid.openid``
        is present in the WSGI environment.
        The name of this key can be adjusted via the ``openid_field`` configuration
        directive.

        The latter is the case when we are coming from the login page where the
        user entered the openid to use.

        ``401`` can come back in any case and then we simply redirect to the login
        form which is configured in the who configuration as ``login_form_url``.

        TODO: make the environment key to check also configurable in the challenge_decider.

        For the OpenID flow check `the OpenID library documentation 
        <http://openidenabled.com/files/python-openid/docs/2.2.1/openid.consumer.consumer-module.html>`_

        """

        request = Request(environ)
        
        # check for the field present, if not redirect to login_form
        if not request.params.has_key(self.openid_field):
            # redirect to login_form
            res = Response()
            res.status = 302
            res.location = self.login_form_url+"?%s=%s" %(self.came_from_field, request.url)
            return res

        # now we have an openid from the user in the request 
        openid_url = request.params[self.openid_field]
        environ['repoze.who.logger'].debug('starting openid request for : %s ' %openid_url)       

        try:
        # we create a new Consumer and start the discovery process for the URL given
        # in the library openid_request is called auth_req btw.
            openid_request = self.get_consumer(environ).begin(openid_url)
        except consumer.DiscoveryFailure, exc:
            # eventually no openid server could be found
            environ[self.error_field] = 'Error in discovery: %s' %exc[0]
            environ['repoze.who.logger'].info('Error in discovery: %s ' %exc[0])     
            return self._redirect_to_loginform(environ)
        except KeyError, exc:
            # TODO: when does that happen, why does plone.openid use "pass" here?
            environ[self.error_field] = 'Error in discovery: %s' %exc[0]
            environ['repoze.who.logger'].info('Error in discovery: %s ' %exc[0])
            return self._redirect_to_loginform(environ)
            return None
           
        # not sure this can still happen but we are making sure.
        # should actually been handled by the DiscoveryFailure exception above
        if openid_request is None:
            environ[self.error_field] = 'No OpenID services found for %s' %openid_url
            environ['repoze.who.logger'].info('No OpenID services found for: %s ' %openid_url)
            return self._redirect_to_loginform(environ)
       
        # we have to tell the openid provider where to send the user after login
        # so we need to compute this from our path and application URL
        # we simply use the URL we are at right now (which is the form)
        # this will be captured by the repoze.who identification plugin later on
        # it will check if some valid openid response is coming back
        # trust_root is the URL (realm) which will be presented to the user
        # in the login process and should be your applications url
        # TODO: make this configurable?
        # return_to is the actual URL to be used for returning to this app
        return_to = request.path_url # we return to this URL here
        trust_root = request.application_url
        environ['repoze.who.logger'].debug('setting return_to URL to : %s ' %return_to)
        
        # TODO: usually you should check openid_request.shouldSendRedirect()
        # but this might say you have to use a form redirect and I don't get why
        # so we do the same as plone.openid and ignore it.

        # TODO: we might also want to give the application some way of adding
        # extensions to this message.
        redirect_url = openid_request.redirectURL(trust_root, return_to) 
        # # , immediate=False)
        res = Response()
        res.status = 302
        res.location = redirect_url
        environ['repoze.who.logger'].debug('redirecting to : %s ' %redirect_url)

        # now it's redirecting and might come back via the identify() method
        # from the openid provider once the user logged in there.
        return res
        
    def _redirect_to_loginform(self, environ={}):
        """redirect the user to the login form"""
        res = Response()
        res.status = 302
        q=''
        ef = environ.get(self.error_field, None)
        if ef is not None:
                q='?%s=%s' %(self.error_field, ef)
        res.location = self.login_form_url+q
        return res
        
                
    # IAuthenticator
    def authenticate(self, environ, identity):
        """dummy authenticator
        
        This takes the openid found and uses it as the userid. Normally you would want
        to take the openid and search a user for it to map maybe multiple openids to a user.
        This means for you to simply implement something similar to this. 
        
        """
        if identity.has_key("repoze.who.plugins.openid.userid"):
                environ['repoze.who.logger'].info('authenticated : %s ' %identity['repoze.who.plugins.openid.userid'])
                return identity.get('repoze.who.plugins.openid.userid')


    def __repr__(self):
        return '<%s %s>' % (self.__class__.__name__, id(self))




# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2008-2009, Gustavo Narea <me@gustavonarea.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the BSD-like license at
# http://www.repoze.org/LICENSE.txt.  A copy of the license should accompany
# this distribution.  THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL
# EXPRESS OR IMPLIED WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO,
# THE IMPLIED WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND
# FITNESS FOR A PARTICULAR PURPOSE
#
##############################################################################

"""Stuff required to setup the test database."""

import os

from sqlalchemy import *
from sqlalchemy.orm import *
from cStringIO import StringIO
from cgi import FieldStorage

from fixture.sa_model import init_model, DBSession, metadata, User, Member

engine = create_engine(os.environ.get('DBURL', 'sqlite://'))

def setup_database():
    init_model(engine)
    teardownDatabase()
    metadata.create_all(engine)
    
    # Creating users

    user = User()
    user.user_name = u'rms'
    user.password = u'freedom'
    DBSession.add(user)

    user = User()
    user.user_name = u'linus'
    user.password = u'linux'
    DBSession.add(user)

    user = User()
    user.user_name = u'sballmer'
    user.password = u'developers'
    DBSession.add(user)

    # Plus a couple of users without groups
    user = User()
    user.user_name = u'guido'
    user.password = u'phytonic'
    DBSession.add(user)

    user = User()
    user.user_name = u'rasmus'
    user.password = u'php'
    DBSession.add(user)

    DBSession.commit()

def setup_database_with_translations():
    init_model(engine)
    teardownDatabase()
    metadata.create_all(engine)
    
    # Creating members

    member = Member()
    member.member_name = u'rms'
    member.password = u'freedom'
    DBSession.add(member)

    member = Member()
    member.member_name = u'linus'
    member.password = u'linux'
    DBSession.add(member)

    member = Member()
    member.member_name = u'sballmer'
    member.password = u'developers'
    DBSession.add(member)

    # Plus a couple of members without groups
    member = Member()
    member.member_name = u'guido'
    member.password = u'phytonic'
    DBSession.add(member)

    member = Member()
    member.member_name = u'rasmus'
    member.password = u'php'
    DBSession.add(member)

    DBSession.commit()


def teardownDatabase():
    DBSession.rollback()
    metadata.drop_all(engine)




"""The application's model objects"""

from zope.sqlalchemy import ZopeTransactionExtension
from sqlalchemy.orm import scoped_session, sessionmaker
#from sqlalchemy import MetaData
from sqlalchemy.ext.declarative import declarative_base


# Global session manager.  DBSession() returns the session object
# appropriate for the current web request.
maker = sessionmaker(autoflush=True, autocommit=False,
                     extension=ZopeTransactionExtension())
DBSession = scoped_session(maker)

# By default, the data model is defined with SQLAlchemy's declarative
# extension, but if you need more control, you can switch to the traditional
# method.
DeclarativeBase = declarative_base()

# Global metadata.
# The default metadata is the one from the declarative base.
metadata = DeclarativeBase.metadata
# If you have multiple databases with overlapping table names, you'll need a
# metadata for each database. Feel free to rename 'metadata2'.
#metadata2 = MetaData()

#####
# Generally you will not want to define your table's mappers, and data objects
# here in __init__ but will want to create modules them in the model directory
# and import them at the bottom of this file.  
#
######

def init_model(engine):
    """Call me before using any of the tables or classes in the model."""

    DBSession.configure(bind=engine)
    
    # If you are using reflection to introspect your database and create 
    # table objects for you, your tables must be defined and mapped inside 
    # the init_model function, so that the engine is available if you 
    # use the model outside tg2, you need to make sure this is called before
    # you use the model. 

    #
    # See the following example: 
    
    #global t_reflected
    
    #t_reflected = Table("Reflected", metadata,
    #    autoload=True, autoload_with=engine)

    #mapper(Reflected, t_reflected)

# Import your model modules here. 
from ldapauth.model.identity import User, Group, Permission



try:
    from setuptools import setup, find_packages
except ImportError:
    from ez_setup import use_setuptools
    use_setuptools()
    from setuptools import setup, find_packages

setup(
    name='LDAPAuth',
    version='0.1',
    description='TG2 Demo application for the repoze.who LDAP plugin',
    author='',
    author_email='',
    #url='',
    install_requires=[
        "TurboGears2",
        "ToscaWidgets >= 0.9.1",
        "zope.sqlalchemy",
        "repoze.who.plugins.ldap"
        ],
    packages=find_packages(exclude=['ez_setup']),
    include_package_data=True,
    test_suite='nose.collector',
    tests_require=['webtest', 'beautifulsoup'],
    package_data={'ldapauth': ['i18n/*/LC_MESSAGES/*.mo',
                                 'templates/*/*',
                                 'public/*/*']},
    #message_extractors = {'ldapauth': [
    #        ('**.py', 'python', None),
    #        ('templates/**.mako', 'mako', None),
    #        ('templates/**.html', 'genshi', None),
    #        ('public/**', 'ignore', None)]},
    
    entry_points="""
    [paste.app_factory]
    main = ldapauth.config.middleware:make_app

    [paste.app_install]
    main = pylons.util:PylonsInstaller
    """,
)



import unittest
import StringIO

from repoze.who.tests import encode_multipart_formdata
from StringIO import StringIO

from repoze.who.plugins.openid.classifiers import openid_challenge_decider
from repoze.who.plugins.openid.identification import OpenIdIdentificationPlugin

from consumer import patch_plugin

class ChallengeTest(unittest.TestCase):
    """test the challenge plugin"""

    def setUp(self):
	self.server_response={
		"openid.mode"              : "id_res",
		"nonce"                    : "nonce",
		"openid.identity"          : "http://repoze.myopenid.com",
		"openid.assoc_handle"      : "assoc_handle",
		"openid.return_to"         : "return_to",
		"openid.signed"            : "signed",
		"openid.sig"               : "sig",
		"openid.invalidate_handle" : "invalidate_handle",
            }
	self.plugin = patch_plugin(OpenIdIdentificationPlugin(
					store = None,
					openid_field = 'repoze.whoplugins.openid.openid',
					error_field = '',
					store_file_path='',
					session_name = '',
					login_handler_path = '/login',
					logout_handler_path = '',
					login_form_url = '/login_form',
					logged_in_url = '',
					logged_out_url = '',
					came_from_field = 'came_from',
					rememberer_name = ''
					)
				    )

	environ = {'wsgi.input':'',
	    'wsgi.url_scheme': 'http',
	    'SERVER_NAME': 'localhost',
	    'SERVER_PORT': '8080',
	    'CONTENT_TYPE':'text/html',
	    'CONTENT_LENGTH':0,
	    'REQUEST_METHOD':'POST',
	    'PATH_INFO': '/protected',
	    'QUERY_STRING':'',
	}

        class DummyLogger:
            warnings = []
            debugs = []
            infos = []
            def warn(self, msg):
                self.warnings.append(msg)
            def debug(self, msg):
                self.debugs.append(msg)
            def info(self, msg):
                self.infos.append(msg)
        logger = environ['repoze.who.logger'] = DummyLogger()

	self.environ=environ

    def tearDown(self):
	pass

    def test_challenge_decider(self):
	"""test challenge decider"""

	environ = self.environ
	environ['repoze.whoplugins.openid.openid'] = 'foobar.com'

	# decider takes environ, status, headers
	self.assertEqual(openid_challenge_decider(environ, '200 Ok', {}), True)
	self.assertEqual(openid_challenge_decider({}, '401 Unauthorized', {}), True)
	self.assertEqual(openid_challenge_decider({}, '200 Ok', {}), False)

    def test_challenge_redirect(self):
	"""check if the challenge plugin works if given an openid"""

	# create a form POST response as if we would post the openid
	fields = [('repoze.whoplugins.openid.openid','foobar.com')]
	content_type, body = encode_multipart_formdata(fields)

	environ = self.environ
	environ['wsgi.input'] = StringIO(body)
	environ['REQUEST_METHOD'] = 'POST'
	environ['CONTENT_LENGTH'] = len(body)
	environ['CONTENT_TYPE'] = content_type

	# in this case the plugin has to redirect to the openid provider
	# faked by MockConsumer in this case
	res = self.plugin.challenge(environ, '200 Ok', {}, {})
	self.assertEqual(res.location,'http://someopenidprovider.com/somewhere')
	self.assertEqual(res.status,'302 Found')

    def test_challenge_show_login_form(self):
	"""test if the challenge plugin redirects to the login form"""

	res = self.plugin.challenge(self.environ, '200 Ok', {}, {})
	self.assertEqual(res.location,'/login_form?came_from=http://localhost:8080/protected')
	self.assertEqual(res.status,'302 Found')

    def test_login_form_send(self):
	"""test if the login form data is received and the environment set correctly"""
	fields = [('repoze.whoplugins.openid.openid','foobar.com')]
	content_type, body = encode_multipart_formdata(fields)

	environ = self.environ
	environ['wsgi.input'] = StringIO(body)
	environ['REQUEST_METHOD'] = 'POST'
	environ['CONTENT_LENGTH'] = len(body)
	environ['CONTENT_TYPE'] = content_type
	environ['PATH_INFO'] = '/login'

	identity = self.plugin.identify(environ)
	self.assertEqual(environ.get('repoze.whoplugins.openid.openid',None), 'foobar.com')

    def test_complete_openid_request(self):
	"""test if the openid request completes"""

	environ = self.environ
	environ['PATH_INFO'] = '/login'

	fields = self.server_response.items()
	content_type, body = encode_multipart_formdata(fields)
	environ['wsgi.input'] = StringIO(body)
	environ['REQUEST_METHOD'] = 'POST'
	environ['CONTENT_LENGTH'] = len(body)
	environ['CONTENT_TYPE'] = content_type
	
	identity = self.plugin.identify(environ)
	self.assertEqual(identity['repoze.who.plugins.openid.userid'],'http://repoze.myopenid.com')

    def test_incomplete_openid_request(self):
	"""test if the openid request fails with a wrong identity"""

	environ = self.environ
	environ['PATH_INFO'] = '/login'

	sresp = self.server_response
	sresp['openid.identity'] = ''

	fields = sresp.items()
	content_type, body = encode_multipart_formdata(fields)
	environ['wsgi.input'] = StringIO(body)
	environ['REQUEST_METHOD'] = 'POST'
	environ['CONTENT_LENGTH'] = len(body)
	environ['CONTENT_TYPE'] = content_type
	
	identity = self.plugin.identify(environ)
	self.assertEqual(identity.get('repoze.who.plugins.openid.userid',None),None)

    def test_authenticate(self):
	"""test if the authentication plugin works as well"""
	environ = self.environ
	identity = {'repoze.who.plugins.openid.userid' : 'http://foobar.com'}
	res = self.plugin.authenticate(environ, identity)
	self.assertEqual(res, 'http://foobar.com')



# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2008-2009, Gustavo Narea <me@gustavonarea.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the BSD-like license at
# http://www.repoze.org/LICENSE.txt.  A copy of the license should accompany
# this distribution.  THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL
# EXPRESS OR IMPLIED WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO,
# THE IMPLIED WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND
# FITNESS FOR A PARTICULAR PURPOSE
#
##############################################################################

"""
Tests for the repoze.who SQLAlchemy MD provider.

"""

import unittest

from repoze.who.plugins.sa import SQLAlchemyUserChecker

import databasesetup_sa
from fixture import sa_model


class TestUserChecker(unittest.TestCase):
    """Tests for the user checker"""
    
    def setUp(self):
        databasesetup_sa.setup_database()
        self.plugin = SQLAlchemyUserChecker(sa_model.User, sa_model.DBSession)
    
    def tearDown(self):
        databasesetup_sa.teardownDatabase()
    
    def test_existing_user(self):
        self.assertTrue(self.plugin(u"guido"))
    
    def test_non_existing_user(self):
        self.assertFalse(self.plugin(u"gustavo"))




# -*- coding: utf-8 -*-
"""Test suite for the TG app's models"""

from tg.testutil import DBTest
from sqlalchemy import create_engine

from ldapauth import model

test_database = create_engine("sqlite:///:memory:")

class TestModel(DBTest):
    """The base class for testing models in you TG project."""
    model = model
    database = test_database

    
class TestUser(TestModel):
    """Test case for the User model."""
    
    def setUp(self):
        super(TestUser, self).setUp()
        self.member = model.User()
        self.member.user_name = u"ignucius"
        self.member.email_address = u"ignucius@example.org"
        
    def test_member_creation_username(self):
        """The member constructor must set the user name right"""
        self.assertEqual(self.member.user_name, u"ignucius")
        
    def test_member_creation_email(self):
        """The member constructor must set the email right"""
        self.assertEqual(self.member.email_address, u"ignucius@example.org")
    
    def test_no_permissions_by_default(self):
        """User objects should have no permission by default."""
        self.assertEqual(len(self.member.permissions), 0)
        
    def test_getting_by_email(self):
        """Users should be fetcheable by their email addresses"""
        model.DBSession.add(self.member)
        him = model.User.by_email_address(u"ignucius@example.org")
        self.assertEqual(him, self.member)



# -*- coding: utf-8 -*-
#
# repoze.who.plugins.ldap, LDAP authentication for WSGI applications.
# Copyright (C) 2008 by Gustavo Narea <http://gustavonarea.net/>
#
# This file is part of repoze.who.plugins.ldap
# <http://code.gustavonarea.net/repoze.who.plugins.ldap/>
#
# repoze.who.plugins.ldap is freedomware: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by the
# Free Software Foundation, either version 3 of the License, or any later
# version.
#
# repoze.who.plugins.ldap is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of 
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# repoze.who.plugins.ldap. If not, see <http://www.gnu.org/licenses/>.

# See http://peak.telecommunity.com/DevCenter/setuptools#namespace-packages
try:
    __import__('pkg_resources').declare_namespace(__name__)
except ImportError:
    from pkgutil import extend_path
    __path__ = extend_path(__path__, __name__)



# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2008-2009, Gustavo Narea <me@gustavonarea.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the BSD-like license at
# http://www.repoze.org/LICENSE.txt.  A copy of the license should accompany
# this distribution.  THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL
# EXPRESS OR IMPLIED WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO,
# THE IMPLIED WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND
# FITNESS FOR A PARTICULAR PURPOSE
#
##############################################################################


# See http://peak.telecommunity.com/DevCenter/setuptools#namespace-packages
try:
    __import__('pkg_resources').declare_namespace(__name__)
except ImportError:
    from pkgutil import extend_path
    __path__ = extend_path(__path__, __name__)



"""Main Controller"""

from tg import expose, flash
from pylons import request
from paste.httpexceptions import HTTPUnauthorized

from ldapauth.lib.base import BaseController


class RootController(BaseController):
    @expose('ldapauth.templates.index')
    def index(self):
        return dict()

    @expose('ldapauth.templates.about')
    def about(self):
        if request.environ.get('repoze.who.identity') == None:
            raise HTTPUnauthorized()
        user = request.environ['repoze.who.identity']['repoze.who.userid']
        flash('Your Distinguished Name (DN) is "%s"' % user)
        # Passing the metadata
        metadata = request.environ['repoze.who.identity']
        return dict(metadata=metadata.items())



"""
Sample SQLAlchemy-powered model definition for the repoze.what SQL plugin.

This model definition has been taken from a quickstarted TurboGears 2 project,
but it's absolutely independent of TurboGears.

"""

import md5
import sha
from datetime import datetime

from sqlalchemy import Table, ForeignKey, Column
from sqlalchemy.types import String, Unicode, UnicodeText, Integer, DateTime, \
                             Boolean, Float
from sqlalchemy.orm import relation, backref, synonym

from yourproject.model import DeclarativeBase, metadata, DBSession


# This is the association table for the many-to-many relationship between
# groups and permissions.
group_permission_table = Table('group_permission', metadata,
    Column('group_id', Integer, ForeignKey('group.group_id',
        onupdate="CASCADE", ondelete="CASCADE")),
    Column('permission_id', Integer, ForeignKey('permission.permission_id',
        onupdate="CASCADE", ondelete="CASCADE"))
)

# This is the association table for the many-to-many relationship between
# groups and members - this is, the memberships.
user_group_table = Table('user_group', metadata,
    Column('user_id', Integer, ForeignKey('user.user_id',
        onupdate="CASCADE", ondelete="CASCADE")),
    Column('group_id', Integer, ForeignKey('group.group_id',
        onupdate="CASCADE", ondelete="CASCADE"))
)

# auth model

class Group(DeclarativeBase):
    """An ultra-simple group definition.
    """
    __tablename__ = 'group'

    group_id = Column(Integer, autoincrement=True, primary_key=True)

    group_name = Column(Unicode(16), unique=True)

    users = relation('User', secondary=user_group_table, backref='groups')


class User(DeclarativeBase):
    """Reasonably basic User definition. Probably would want additional
    attributes.
    """
    __tablename__ = 'user'

    user_id = Column(Integer, autoincrement=True, primary_key=True)
    
    user_name = Column(Unicode(16), unique=True)

    _password = Column('password', Unicode(40))

    def _set_password(self, password):
        """encrypts password on the fly."""
        self._password = self.__encrypt_password(password)

    def _get_password(self):
        """returns password"""
        return self._password

    password = synonym('password', descriptor=property(_get_password,
                                                       _set_password))

    def __encrypt_password(self, password):
        """Hash the given password with SHA1.
        
        Edit this method to implement your own algorithm.
        
        """
        hashed_password = password

        if isinstance(password, unicode):
            password_8bit = password.encode('UTF-8')

        else:
            password_8bit = password

        hashed_password = sha.new(password_8bit).hexdigest()

        # make sure the hased password is an UTF-8 object at the end of the
        # process because SQLAlchemy _wants_ a unicode object for Unicode columns
        if not isinstance(hashed_password, unicode):
            hashed_password = hashed_password.decode('UTF-8')

        return hashed_password

    def validate_password(self, password):
        """Check the password against existing credentials.
        this method _MUST_ return a boolean.

        @param password: the password that was provided by the user to
        try and authenticate. This is the clear text version that we will
        need to match against the (possibly) encrypted one in the database.
        @type password: unicode object
        """
        return self.password == self.__encrypt_password(password)


class Permission(DeclarativeBase):
    """A relationship that determines what each Group can do"""
    __tablename__ = 'permission'

    permission_id = Column(Integer, autoincrement=True, primary_key=True)

    permission_name = Column(Unicode(16), unique=True)

    groups = relation(Group, secondary=group_permission_table,
                      backref='permissions')



# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2007, Agendaless Consulting and Contributors.
# Copyright (c) 2008, Florent Aide <florent.aide@gmail.com>.
# Copyright (c) 2008-2009, Gustavo Narea <me@gustavonarea.net>.
# All Rights Reserved.
#
# This software is subject to the provisions of the BSD-like license at
# http://www.repoze.org/LICENSE.txt.  A copy of the license should accompany
# this distribution.  THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL
# EXPRESS OR IMPLIED WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO,
# THE IMPLIED WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND
# FITNESS FOR A PARTICULAR PURPOSE
#
##############################################################################

import os

from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, 'README.txt')).read()
version = open(os.path.join(here, 'VERSION.txt')).readline().rstrip()

setup(name='repoze.who.plugins.sa',
      version=version,
      description=('The repoze.who SQLAlchemy plugin'),
      long_description=README,
      classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Topic :: Database",
        "Topic :: Security",
        ],
      keywords='web application server wsgi sql sqlalchemy elixir ' \
               'authentication repoze',
      author="Gustavo Narea",
      author_email="repoze-dev@lists.repoze.org",
      namespace_packages=['repoze', 'repoze.who', 'repoze.who.plugins'],
      url="http://code.gustavonarea.net/repoze.who.plugins.sa/",
      license="BSD-derived (http://www.repoze.org/LICENSE.txt)",
      packages=find_packages(),
      include_package_data=True,
      zip_safe=False,
      tests_require=[
          'repoze.who >= 1.0.14',
          'coverage',
          'nose',
          'sqlalchemy >= 0.5.0',
          'elixir'],
      install_requires=['repoze.who >= 1.0.14', 'sqlalchemy >= 0.5.0'],
      test_suite="nose.collector",
      entry_points = """\
      """
      )




# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009, Gustavo Narea <me@gustavonarea.net>.
# All Rights Reserved.
#
# This software is subject to the provisions of the BSD-like license at
# http://www.repoze.org/LICENSE.txt.  A copy of the license should accompany
# this distribution.  THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL
# EXPRESS OR IMPLIED WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO,
# THE IMPLIED WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND
# FITNESS FOR A PARTICULAR PURPOSE.
#
##############################################################################

import os

from ez_setup import use_setuptools
use_setuptools()
from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, 'README.txt')).read()
version = open(os.path.join(here, 'VERSION.txt')).readline().rstrip()

setup(name='repoze.who-testutil',
      version=version,
      description=('Test utilities for repoze.who-powered applications'),
      long_description=README,
      classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Topic :: Internet :: WWW/HTTP :: WSGI",
        "Topic :: Security"
        ],
      keywords='web application wsgi authentication testing tests repoze',
      author='Gustavo Narea',
      author_email='repoze-dev@lists.repoze.org',
      namespace_packages = ['repoze', 'repoze.who', 'repoze.who.plugins'],
      url='http://code.gustavonarea.net/repoze.who-testutil/',
      license='BSD-derived (http://www.repoze.org/LICENSE.txt)',
      packages=find_packages(),
      include_package_data=True,
      zip_safe=False,
      tests_require=['repoze.who >= 1.0', 'coverage', 'nose'],
      install_requires=[
        'repoze.who >= 1.0',
        'zope.interface',
        'Paste > 1.7', # Workaround for the buggy setuptools
        'PasteDeploy >= 1.3.3'],
      test_suite='nose.collector',
      entry_points = """\
      """
      )



# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2008-2009, Gustavo Narea <me@gustavonarea.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the BSD-like license at
# http://www.repoze.org/LICENSE.txt.  A copy of the license should accompany
# this distribution.  THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL
# EXPRESS OR IMPLIED WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO,
# THE IMPLIED WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND
# FITNESS FOR A PARTICULAR PURPOSE
#
##############################################################################

"""
Tests for the repoze.who SQLAlchemy authenticator.

"""

import unittest

from zope.interface.verify import verifyClass
from repoze.who.interfaces import IAuthenticator

from repoze.who.plugins.sa import SQLAlchemyAuthenticatorPlugin, \
                                  make_sa_authenticator

import databasesetup_sa, databasesetup_elixir
from fixture import sa_model, elixir_model


class TestAuthenticator(unittest.TestCase):
    """Tests for the authenticator function"""
    
    def setUp(self):
        databasesetup_sa.setup_database()
        self.plugin = SQLAlchemyAuthenticatorPlugin(sa_model.User, 
                                                    sa_model.DBSession)
    
    def tearDown(self):
        databasesetup_sa.teardownDatabase()

    def test_implements(self):
        verifyClass(IAuthenticator, SQLAlchemyAuthenticatorPlugin, 
                    tentative=True)
        
    def test_no_identity(self):
        identity = {}
        self.assertEqual(None, self.plugin.authenticate(None, identity))
        
    def test_incomplete_credentials(self):
        identity = {'login': 'rms'}
        self.assertEqual(None, self.plugin.authenticate(None, identity))
        identity = {'password': 'freedom'}
        self.assertEqual(None, self.plugin.authenticate(None, identity))
        
    def test_no_match(self):
        identity = {'login': u'gustavo', 'password': u'narea'}
        self.assertEqual(None, self.plugin.authenticate(None, identity))
        
    def test_match(self):
        identity = {'login': u'rms', 'password': u'freedom'}
        self.assertEqual(u'rms', self.plugin.authenticate(None, identity))


class TestAuthenticatorWithTranslations(unittest.TestCase):
    """Tests for the authenticator function"""
    
    def setUp(self):
        databasesetup_sa.setup_database_with_translations()
    
    def tearDown(self):
        databasesetup_sa.teardownDatabase()
    
    def test_it(self):
        self.plugin = SQLAlchemyAuthenticatorPlugin(sa_model.Member, 
                                                    sa_model.DBSession)
        # Updating the translations...
        self.plugin.translations['user_name'] = 'member_name'
        self.plugin.translations['validate_password'] = 'verify_pass'
        # Testing it...
        identity = {'login': u'rms', 'password': u'freedom'}
        self.assertEqual(u'rms', self.plugin.authenticate(None, identity))


class TestAuthenticatorWithElixir(TestAuthenticator):
    
    def setUp(self):
        databasesetup_elixir.setup_database()
        self.plugin = SQLAlchemyAuthenticatorPlugin(elixir_model.User,
                                            elixir_model.DBSession)
    
    def tearDown(self):
        databasesetup_elixir.teardownDatabase()


class TestAuthenticatorMaker(unittest.TestCase):
    
    def setUp(self):
        databasesetup_sa.setup_database()
    
    def tearDown(self):
        databasesetup_sa.teardownDatabase()
    
    def test_simple_call(self):
        user_class = 'tests.fixture.sa_model:User'
        dbsession = 'tests.fixture.sa_model:DBSession'
        authenticator = make_sa_authenticator(user_class, dbsession)
        self.assertTrue(isinstance(authenticator, 
                                   SQLAlchemyAuthenticatorPlugin))
    
    def test_no_user_class(self):
        dbsession = 'tests.fixture.sa_model:DBSession'
        self.assertRaises(ValueError, make_sa_authenticator, None, dbsession)
    
    def test_no_dbsession(self):
        user_class = 'tests.fixture.sa_model:User'
        self.assertRaises(ValueError, make_sa_authenticator, user_class)
    
    def test_username_translation(self):
        user_class = 'tests.fixture.sa_model:User'
        dbsession = 'tests.fixture.sa_model:DBSession'
        username_translation = 'username'
        authenticator = make_sa_authenticator(user_class, dbsession,
                                              username_translation)
        self.assertTrue(isinstance(authenticator, 
                                   SQLAlchemyAuthenticatorPlugin))
        self.assertEqual(username_translation,
                         authenticator.translations['user_name'])
    
    def test_passwd_validator_translation(self):
        user_class = 'tests.fixture.sa_model:User'
        dbsession = 'tests.fixture.sa_model:DBSession'
        password_validator_translation = 'verify_pass'
        authenticator = make_sa_authenticator(
            user_class,
            dbsession,
            validate_password_translation=password_validator_translation)
        self.assertTrue(isinstance(authenticator, 
                                   SQLAlchemyAuthenticatorPlugin))
        self.assertEqual(password_validator_translation,
                         authenticator.translations['validate_password'])



# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2008-2009, Gustavo Narea <me@gustavonarea.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the BSD-like license at
# http://www.repoze.org/LICENSE.txt.  A copy of the license should accompany
# this distribution.  THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL
# EXPRESS OR IMPLIED WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO,
# THE IMPLIED WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND
# FITNESS FOR A PARTICULAR PURPOSE
#
##############################################################################

"""
SQLAlchemy plugin for repoze.who.

TODO: Write a function that configures the three plugins in one go.

"""

from zope.interface import implements
from repoze.who.interfaces import IAuthenticator, IMetadataProvider
from repoze.who.utils import resolveDotted
from sqlalchemy.orm.exc import NoResultFound, MultipleResultsFound


__all__ = ("SQLAlchemyAuthenticatorPlugin", "SQLAlchemyUserMDPlugin",
           "SQLAlchemyUserChecker", "make_sa_authenticator",
           "make_sa_user_mdprovider")


class _BaseSQLAlchemyPlugin(object):
    
    default_translations = {'user_name': "user_name"}
    
    def __init__(self, user_class, dbsession):
        """
        Setup the plugin.
    
        :param user_class: The SQLAlchemy/Elixir class for the users.
        :param session: The SQLAlchemy/Elixir session.
        
        """
        self.user_class = user_class
        self.dbsession = dbsession
        self.translations = self.default_translations.copy()
    
    def get_user(self, username):
        # Getting a translation:
        username_attr = getattr(self.user_class,
                                self.translations['user_name'])
        
        query = self.dbsession.query(self.user_class)
        query = query.filter(username_attr==username)
        
        try:
            return query.one()
        except (NoResultFound, MultipleResultsFound):
            # As recommended in the docs for repoze.who, it's important to
            # verify that there's only _one_ matching userid.
            return None


class SQLAlchemyUserChecker(_BaseSQLAlchemyPlugin):
    """
    User existence checker for
    :class:`repoze.who.plugins.auth_tkt.AuthTktCookiePlugin`.
    
    Example::
    
        from repoze.who.plugins.sa import SQLAlchemyUserChecker
        from yourcoolproject.model import User, DBSession
        
        checker = SQLAlchemyUserChecker(User, DBSession)
    
    This plugin assumes that the user name is kept in the ``user_name``
    attribute of the users' class. If you don't want to call it that way, then
    you have to "translate" it as in the sample below::
    
        # You have User.username instead of User.user_name:
        checker.translations['user_name'] = 'username'
    
    """
    
    def __call__(self, username):
        """
        Check whether a user account identified by ``username`` exists.
        
        :param username: The user account's id to be verified.
        :type username: basestring
        :return: Whether it exists or not.
        :rtype: bool
        
        """
        if self.get_user(username):
            return True
        return False


#{ repoze.who plugins


class SQLAlchemyAuthenticatorPlugin(_BaseSQLAlchemyPlugin):
    """
    :mod:`repoze.who` authenticator for SQLAlchemy models.
    
    Example::
    
        from repoze.who.plugins.sa import SQLAlchemyAuthenticatorPlugin
        from yourcoolproject.model import User, DBSession
        
        authenticator = SQLAlchemyAuthenticatorPlugin(User, DBSession)
    
    This plugin assumes that the user name is kept in the ``user_name``
    attribute of the users' class, as well as that such a class has a method
    that verifies the user's password against the password provided through the
    login form (it receives the password to be verified as the only argument
    and such method is assumed to be called ``validate_password``).
    
    If you don't want to call the attributes above as ``user_name`` and/or
    ``validate_password``, respectively, then you have to "translate" them as
    in the sample below::
    
        # You have User.username instead of User.user_name:
        authenticator.translations['user_name'] = 'username'
        
        # You have User.verify_password instead of User.validate_password:
        authenticator.translations['validate_password'] = 'verify_password'
    
    .. note::
    
        If you want to configure this authenticator from an ``ini`` file, use
        :func:`make_sa_authenticator`.
    
    """
    
    implements(IAuthenticator)
    
    default_translations = _BaseSQLAlchemyPlugin.default_translations.copy()
    default_translations['validate_password'] = "validate_password"

    # IAuthenticator
    def authenticate(self, environ, identity):
        if not ("login" in identity and "password" in identity):
            return None
        
        user = self.get_user(identity['login'])
        
        if user:
            validator = getattr(user, self.translations['validate_password'])
            if validator(identity['password']):
                return identity['login']


class SQLAlchemyUserMDPlugin(_BaseSQLAlchemyPlugin):
    """
    :mod:`repoze.who` metadata provider that loads the SQLAlchemy-powered
    object for the current user.
    
    It loads the object into ``identity['user']``.
    
    Example::
    
        from repoze.who.plugins.sa import SQLAlchemyUserMDPlugin
        from yourcoolproject.model import User, DBSession
        
        mdprovider = SQLAlchemyUserMDPlugin(User, DBSession)
    
    This plugin assumes that the user name is kept in the ``user_name``
    attribute of the users' class. If you don't want to call the attribute
    above as ``user_name``, then you have to "translate" it as in the sample 
    below::
    
        # You have User.username instead of User.user_name:
        mdprovider.translations['user_name'] = 'username'
    
    .. note::
    
        If you want to configure this plugin from an ``ini`` file, use
        :func:`make_sa_user_mdprovider`.
    
    """
    
    implements(IMetadataProvider)
    
    def add_metadata(self, environ, identity):
        identity['user'] = self.get_user(identity['repoze.who.userid'])


#{ Functions to instantiate the plugins from a Paste configuration


def _base_plugin_maker(user_class=None, dbsession=None):
    """
    Turn ``userclass`` and ``dbsession`` into Python objects.
    
    """
    
    if user_class is None:
        raise ValueError("user_class must not be None")
    if dbsession is None:
        raise ValueError("dbsession must not be None")
    return resolveDotted(user_class), resolveDotted(dbsession)


def make_sa_authenticator(user_class=None, dbsession=None, 
                          user_name_translation=None, 
                          validate_password_translation=None):
    """
    Configure :class:`SQLAlchemyAuthenticatorPlugin`.
    
    :param user_class: The SQLAlchemy/Elixir class for the users.
    :type user_class: str
    :param dbsession: The SQLAlchemy/Elixir session.
    :type dbsession: str
    :param user_name_translation: The translation for ``user_name``, if any.
    :type user_name_translation: str
    :param validate_password_translation: The translation for 
        ``validate_password``, if any.
    :type validate_password_translation: str
    :return: The authenticator.
    :rtype: SQLAlchemyAuthenticatorPlugin
    
    Example from an ``*.ini`` file::
    
        # ...
        [plugin:sa_auth]
        use = repoze.who.plugins.sa:make_sa_authenticator
        user_class = yourcoolproject.model:User
        dbsession = yourcoolproject.model:DBSession
        # ...
    
    Or, if you need translations::
    
        # ...
        [plugin:sa_auth]
        use = repoze.who.plugins.sa:make_sa_authenticator
        user_class = yourcoolproject.model:User
        dbsession = yourcoolproject.model:DBSession
        user_name_translation = username
        validate_password_translation = verify_password
        # ...
    
    """
    
    user_model, dbsession_object = _base_plugin_maker(user_class, dbsession)
    
    authenticator = SQLAlchemyAuthenticatorPlugin(user_model, dbsession_object)
    
    if user_name_translation:
        authenticator.translations['user_name'] = user_name_translation
    if validate_password_translation:
        authenticator.translations['validate_password'] = \
            validate_password_translation
    
    return authenticator


def make_sa_user_mdprovider(user_class=None, dbsession=None, 
                            user_name_translation=None):
    """
    Configure :class:`SQLAlchemyUserMDPlugin`.
    
    :param user_class: The SQLAlchemy/Elixir class for the users.
    :type user_class: str
    :param dbsession: The SQLAlchemy/Elixir session.
    :type dbsession: str
    :param user_name_translation: The translation for ``user_name``, if any.
    :type user_name_translation: str
    :return: The metadata provider.
    :rtype: SQLAlchemyUserMDPlugin
    
    Example from an ``*.ini`` file::
    
        # ...
        [plugin:sa_md]
        use = repoze.who.plugins.sa:make_sa_user_mdprovider
        user_class = yourcoolproject.model:User
        dbsession = yourcoolproject.model:DBSession
        # ...
    
    Or, if you need translations::
    
        # ...
        [plugin:sa_md]
        use = repoze.who.plugins.sa:make_sa_user_mdprovider
        user_class = yourcoolproject.model:User
        dbsession = yourcoolproject.model:DBSession
        user_name_translation = username
        # ...
    
    """
    
    user_model, dbsession_object = _base_plugin_maker(user_class, dbsession)
    
    mdprovider = SQLAlchemyUserMDPlugin(user_model, dbsession_object)
    
    if user_name_translation:
        mdprovider.translations['user_name'] = user_name_translation
    
    return mdprovider


#}



# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009, Gustavo Narea <me@gustavonarea.net>.
# All Rights Reserved.
#
# This software is subject to the provisions of the BSD-like license at
# http://www.repoze.org/LICENSE.txt.  A copy of the license should accompany
# this distribution.  THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL
# EXPRESS OR IMPLIED WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO,
# THE IMPLIED WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND
# FITNESS FOR A PARTICULAR PURPOSE.
#
##############################################################################

"""Collection of :mod:`repoze.who` friendly forms"""

from urlparse import urlparse, urlunparse
from urllib import urlencode
try:
    from urlparse import parse_qs
except ImportError:
    from cgi import parse_qs

from paste.httpexceptions import HTTPFound, HTTPUnauthorized
from paste.request import construct_url, parse_dict_querystring, parse_formvars
from paste.response import replace_header, header_value
from zope.interface import implements

from repoze.who.interfaces import IChallenger, IIdentifier

__all__ = ['FriendlyFormPlugin']


class FriendlyFormPlugin(object):
    """
    :class:`RedirectingFormPlugin 
    <repoze.who.plugins.form.RedirectingFormPlugin>`-like form plugin with
    more features.
    
    It is like ``RedirectingFormPlugin``, but provides us with the following
    features:
    
    * Users are not challenged on logout, unless the referrer URL is a
      private one (but that's up to the application).
    * Developers may define post-login and/or post-logout pages.
    * In the login URL, the amount of failed logins is available in the
      environ. It's also increased by one on every login try. This counter 
      will allow developers not using a post-login page to handle logins that
      fail/succeed.
    
    You should keep in mind that if you're using a post-login or a post-logout
    page, that page will receive the referrer URL as a query string variable
    whose name is "came_from".
    
    """
    implements(IChallenger, IIdentifier)
    
    def __init__(self, login_form_url, login_handler_path, post_login_url,
                 logout_handler_path, post_logout_url, rememberer_name,
                 login_counter_name=None):
        """
        
        :param login_form_url: The URL/path where the login form is located.
        :type login_form_url: str
        :param login_handler_path: The URL/path where the login form is
            submitted to (where it is processed by this plugin).
        :type login_handler_path: str
        :param post_login_url: The URL/path where the user should be redirected
            to after login (even if wrong credentials were provided).
        :type post_login_url: str
        :param logout_handler_path: The URL/path where the user is logged out.
        :type logout_handler_path: str
        :param post_logout_url: The URL/path where the user should be
            redirected to after logout.
        :type post_logout_url: str
        :param rememberer_name: The name of the repoze.who identifier which
            acts as rememberer.
        :type rememberer_name: str
        :param login_counter_name: The name of the query string variable which
            will represent the login counter.
        :type login_counter_name: str
        
        The login counter variable's name will be set to ``__logins`` if
        ``login_counter_name`` equals None.
        
        """
        self.login_form_url = login_form_url
        self.login_handler_path = login_handler_path
        self.post_login_url = post_login_url
        self.logout_handler_path = logout_handler_path
        self.post_logout_url = post_logout_url
        self.rememberer_name = rememberer_name
        self.login_counter_name = login_counter_name
        if not login_counter_name:
            self.login_counter_name = '__logins'
    
    # IIdentifier
    def identify(self, environ):
        """
        Override the parent's identifier to introduce a login counter
        (possibly along with a post-login page) and load the login counter into
        the ``environ``.
        
        """
        
        path_info = environ['PATH_INFO']
        script_name = environ.get('SCRIPT_NAME') or '/'
        query = parse_dict_querystring(environ)
        
        if path_info == self.login_handler_path:
            ## We are on the URL where repoze.who processes authentication. ##
            # Let's append the login counter to the query string of the
            # "came_from" URL. It will be used by the challenge below if
            # authorization is denied for this request.
            form = parse_formvars(environ)
            form.update(query)
            try:
                login = form['login']
                password = form['password']
                credentials = {
                    'login': form['login'],
                    'password': form['password']
                    }
            except KeyError:
                credentials = None
            referer = environ.get('HTTP_REFERER', script_name)
            destination = form.get('came_from', referer)
            
            if self.post_login_url:
                # There's a post-login page, so we have to replace the
                # destination with it.
                destination = self._get_full_path(self.post_login_url,
                                                  environ)
                if 'came_from' in query:
                    # There's a referrer URL defined, so we have to pass it to
                    # the post-login page as a GET variable.
                    destination = self._insert_qs_variable(destination,
                                                           'came_from',
                                                           query['came_from'])
            failed_logins = self._get_logins(environ, True)
            new_dest = self._set_logins_in_url(destination, failed_logins)
            environ['repoze.who.application'] = HTTPFound(new_dest)
            return credentials

        elif path_info == self.logout_handler_path:
            ##    We are on the URL where repoze.who logs the user out.    ##
            form = parse_formvars(environ)
            form.update(query)
            referer = environ.get('HTTP_REFERER', script_name)
            came_from = form.get('came_from', referer)
            # set in environ for self.challenge() to find later
            environ['came_from'] = came_from
            environ['repoze.who.application'] = HTTPUnauthorized()
            return None
            
        elif path_info == self.login_form_url or self._get_logins(environ):
            ##  We are on the URL that displays the from OR any other page  ##
            ##   where the login counter is included in the query string.   ##
            # So let's load the counter into the environ and then hide it from
            # the query string (it will cause problems in frameworks like TG2,
            # where this unexpected variable would be passed to the controller)
            environ['repoze.who.logins'] = self._get_logins(environ, True)
            # Hiding the GET variable in the environ:
            if self.login_counter_name in query:
                del query[self.login_counter_name]
                environ['QUERY_STRING'] = urlencode(query, doseq=True)
    
    # IChallenger
    def challenge(self, environ, status, app_headers, forget_headers):
        """
        Override the parent's challenge to avoid challenging the user on
        logout, introduce a post-logout page and/or pass the login counter 
        to the login form.
        
        """
        url_parts = list(urlparse(self.login_form_url))
        query = url_parts[4]
        query_elements = parse_qs(query)
        came_from = environ.get('came_from', construct_url(environ))
        query_elements['came_from'] = came_from
        url_parts[4] = urlencode(query_elements, doseq=True)
        login_form_url = urlunparse(url_parts)
        login_form_url = self._get_full_path(login_form_url, environ)
        destination = login_form_url
        # Configuring the headers to be set:
        cookies = [(h,v) for (h,v) in app_headers if h.lower() == 'set-cookie']
        headers = forget_headers + cookies
        
        if environ['PATH_INFO'] == self.logout_handler_path:
            # Let's log the user out without challenging.
            came_from = environ.get('came_from')
            if self.post_logout_url:
                # Redirect to a predefined "post logout" URL.
                destination = self._get_full_path(self.post_logout_url,
                                                  environ)
                if came_from:
                    destination = self._insert_qs_variable(
                                  destination, 'came_from', came_from)
            else:
                # Redirect to the referrer URL.
                script_name = environ.get('SCRIPT_NAME', '')
                destination = came_from or script_name or '/'
        
        elif 'repoze.who.logins' in environ:
            # Login failed! Let's redirect to the login form and include
            # the login counter in the query string
            environ['repoze.who.logins'] += 1
            # Re-building the URL:
            destination = self._set_logins_in_url(destination,
                                                  environ['repoze.who.logins'])
        
        return HTTPFound(destination, headers=headers)

    # IIdentifier
    def remember(self, environ, identity):
        rememberer = self._get_rememberer(environ)
        return rememberer.remember(environ, identity)

    # IIdentifier
    def forget(self, environ, identity):
        rememberer = self._get_rememberer(environ)
        return rememberer.forget(environ, identity)
    
    def _get_rememberer(self, environ):
        rememberer = environ['repoze.who.plugins'][self.rememberer_name]
        return rememberer
    
    def _get_full_path(self, path, environ):
        """
        Return the full path to ``path`` by prepending the SCRIPT_NAME.
        
        If ``path`` is a URL, do nothing.
        
        """
        if path.startswith('/'):
            path = environ.get('SCRIPT_NAME', '') + path
        return path
    
    def _get_logins(self, environ, force_typecast=False):
        """
        Return the login counter from the query string in the ``environ``.
        
        If it's not possible to convert it into an integer and  
        ``force_typecast`` is ``True``, it will be set to zero (int(0)). 
        Otherwise, it will be ``None`` or an string.
        
        """
        variables = parse_dict_querystring(environ)
        failed_logins = variables.get(self.login_counter_name)
        if force_typecast:
            try:
                failed_logins = int(failed_logins)
            except (ValueError, TypeError):
                failed_logins = 0
        return failed_logins
    
    def _set_logins_in_url(self, url, logins):
        """
        Insert the login counter variable with the ``logins`` value into
        ``url`` and return the new URL.
        
        """
        return self._insert_qs_variable(url, self.login_counter_name, logins)
    
    def _insert_qs_variable(self, url, var_name, var_value):
        """
        Insert the variable ``var_name`` with value ``var_value`` in the query
        string of ``url`` and return the new URL.
        
        """
        url_parts = list(urlparse(url))
        query_parts = parse_qs(url_parts[4])
        query_parts[var_name] = var_value
        url_parts[4] = urlencode(query_parts, doseq=True)
        return urlunparse(url_parts)
    
    def __repr__(self):
        return '<%s %s>' % (self.__class__.__name__, id(self))



# -*- coding: utf-8 -*-
#
# repoze.who.plugins.ldap, LDAP authentication for WSGI applications.
# Copyright (C) 2008 by Gustavo Narea <http://gustavonarea.net/>
#
# This file is part of repoze.who.plugins.ldap
# <http://code.gustavonarea.net/repoze.who.plugins.ldap/>
#
# repoze.who.plugins.ldap is freedomware: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by the
# Free Software Foundation, either version 3 of the License, or any later
# version.
#
# repoze.who.plugins.ldap is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of 
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# repoze.who.plugins.ldap. If not, see <http://www.gnu.org/licenses/>.

"""repoze.who LDAP plugin

U{repoze.who.plugins.ldap <http://code.gustavonarea.net/repoze.who.plugins.ldap/>}
is a Python package that provides U{repoze.who 
<http://static.repoze.org/whodocs/>} plugins for U{LDAP 
<http://en.wikipedia.org/wiki/Lightweight_Directory_Access_Protocol>}
authentication in U{WSGI 
<http://en.wikipedia.org/wiki/Web_Server_Gateway_Interface>}
applications. It can be used with any LDAP server and any WSGI framework (or no
framework at all).

For information on how to get started, you may want to visit its web site:
U{http://code.gustavonarea.net/repoze.who.plugins.ldap/}

G{packagetree}

"""

import ldap

from repoze.who.plugins.ldap.plugins import LDAPAuthenticatorPlugin, \
                                            LDAPAttributesPlugin

__all__ = ['LDAPAuthenticatorPlugin', 'LDAPAttributesPlugin']



# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2008-2009, Gustavo Narea <me@gustavonarea.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the BSD-like license at
# http://www.repoze.org/LICENSE.txt.  A copy of the license should accompany
# this distribution.  THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL
# EXPRESS OR IMPLIED WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO,
# THE IMPLIED WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND
# FITNESS FOR A PARTICULAR PURPOSE
#
##############################################################################

"""
Tests for the repoze.who SQLAlchemy MD provider.

"""

import unittest

from zope.interface.verify import verifyClass
from repoze.who.interfaces import IMetadataProvider

from repoze.who.plugins.sa import SQLAlchemyUserMDPlugin, \
                                  make_sa_user_mdprovider

import databasesetup_sa
from fixture import sa_model


class TestMDProvider(unittest.TestCase):
    """Tests for the authenticator function"""
    
    def setUp(self):
        databasesetup_sa.setup_database()
        self.plugin = SQLAlchemyUserMDPlugin(sa_model.User, sa_model.DBSession)
    
    def tearDown(self):
        databasesetup_sa.teardownDatabase()

    def test_implements(self):
        verifyClass(IMetadataProvider, SQLAlchemyUserMDPlugin, tentative=True)
        
    def test_it(self):
        user = sa_model.DBSession.query(sa_model.User).\
               filter(sa_model.User.user_name==u'rms').one()
        identity = {'repoze.who.userid': user.user_name}
        expected_identity = {
            'repoze.who.userid': user.user_name,
            'user': user}
        self.plugin.add_metadata(None, identity)
        self.assertEqual(identity, expected_identity)


class TestMDProviderWithTranslations(unittest.TestCase):
    """Tests for the translation functionality"""
    
    def setUp(self):
        databasesetup_sa.setup_database_with_translations()
    
    def tearDown(self):
        databasesetup_sa.teardownDatabase()
    
    def test_it(self):
        self.plugin = SQLAlchemyUserMDPlugin(sa_model.Member,
                                             sa_model.DBSession)
        # Updating the translations...
        self.plugin.translations['user_name'] = 'member_name'
        # Testing it...
        member = sa_model.DBSession.query(sa_model.Member).\
                 filter(sa_model.Member.member_name==u'rms').one()
        identity = {'repoze.who.userid': member.member_name}
        expected_identity = {
            'repoze.who.userid': member.member_name,
            'user': member}
        self.plugin.add_metadata(None, identity)
        self.assertEqual(expected_identity, identity)


class TestMDProviderMaker(unittest.TestCase):
    
    def setUp(self):
        databasesetup_sa.setup_database()
    
    def tearDown(self):
        databasesetup_sa.teardownDatabase()
    
    def test_simple_call(self):
        user_class = 'tests.fixture.sa_model:User'
        dbsession = 'tests.fixture.sa_model:DBSession'
        mdprovider = make_sa_user_mdprovider(user_class, dbsession)
        self.assertTrue(isinstance(mdprovider, SQLAlchemyUserMDPlugin))
    
    def test_no_user_class(self):
        dbsession = 'tests.fixture.sa_model:DBSession'
        self.assertRaises(ValueError, make_sa_user_mdprovider, None, dbsession)
    
    def test_no_dbsession(self):
        user_class = 'tests.fixture.sa_model:User'
        self.assertRaises(ValueError, make_sa_user_mdprovider, user_class)
    
    def test_username_translation(self):
        user_class = 'tests.fixture.sa_model:User'
        dbsession = 'tests.fixture.sa_model:DBSession'
        username_translation = 'username'
        mdprovider = make_sa_user_mdprovider(user_class, dbsession,
                                             username_translation)
        self.assertTrue(isinstance(mdprovider, SQLAlchemyUserMDPlugin))
        self.assertEqual(username_translation,
                         mdprovider.translations['user_name'])
